<?php
/**
 * DESCRIPTION
 *
 * @package Content
 * @since 0.1
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\Model;

class ContentModel extends Model {

	public static $ExportGroup = 't:STRING_SITE_CONTENT';
	
	public static $Schema = [
		'id' => [
			'type' => Model::ATT_TYPE_UUID,
			'required' => true,
			'null' => false,
		],
		'site' => [
			'type' => Model::ATT_TYPE_SITE,
			'formtype' => 'system',
		],
		'nickname' => [
			'type' => Model::ATT_TYPE_STRING,
			'comment' => 'Cache of the linked page title',
			'required' => false,
			'formtype' => 'disabled',
		],
		'editpermissions' => [
			'type' => Model::ATT_TYPE_STRING,
			'default' => '!*',
			'form' => [
				'type' => 'access',
				'group' => 't:STRING_ACCESS_AND_ADVANCED',
			]
		],
		'created' => [
			'type' => Model::ATT_TYPE_CREATED,
			'null' => false,
		],
		'updated' => [
			'type' => Model::ATT_TYPE_UPDATED,
			'null' => false,
		],
	];
	
	public static $Indexes = [
		'primary' => ['id'],
	];
	
    public function __construct(...$keys) {
		$this->_linked = [
			'Page' => [
				'link' => Model::LINK_HASONE,
				'on' => 'baseurl',
			],
		];
		
		parent::__construct(...$keys);
	}
	
	public function get($k, $format = null) {
		switch($k){
			case 'baseurl':
				return '/content/view/' . $this->_columns['id']->valueTranslated;
			default:
				return parent::get($k, $format);
		}
	}
}
