<?php
/**
 * File for class ViewMeta_favicon definition in the coreplus project
 * 
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20140323.0350
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\ViewMeta;

use Core\ConfigHandler;
use Core\Filestore\Factory;
use Favicon\FaviconHelper;


/**
 * A short teaser of what ViewMeta_favicon does.
 *
 * More lengthy description of what ViewMeta_favicon does and why it's fantastic.
 *
 * <h3>Usage Examples</h3>
 *
 *
 * @todo Write documentation for ViewMeta_favicon
 * <h4>Example 1</h4>
 * <p>Description 1</p>
 * <code>
 * // Some code for example 1
 * $a = $b;
 * </code>
 *
 *
 * <h4>Example 2</h4>
 * <p>Description 2</p>
 * <code>
 * // Some code for example 2
 * $b = $a;
 * </code>
 *
 * 
 * @author Charlie Powell <charlie@evalagency.com>
 *
 */
class Favicon extends Base {
	public function fetch() : array {
		$data = [];

		$sizes = FaviconHelper::GetFaviconSizes();

		$appleSizes = [
			'180', '152', '120', '76', '60'
		];
		
		$standardSizes = [
			'512', '256', '196', '192', '96', '64', '48', '32', '16'
		];
		
		foreach($sizes as $s => $file){

			$sxs = $s . 'x' . $s;
			$url = $file->getPreviewURL($sxs . '!');

			if(in_array($s, $appleSizes)){
				$data[] = '<link rel="apple-touch-icon" sizes="' . $sxs . '" href="' . $url . '"/>';
			}

			if(in_array($s, $standardSizes)){
				$data[] = '<link rel="shortcut icon" sizes="' . $sxs . '" href="' . $url . '"/>';
			}
		}
		
		return $data;
	}
} 
