/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */


Navigator2 = function(){
	let type,
		$navigator = document.getElementById('media-manager-navigator'),
		$listing = document.getElementById('media-manager-listing'),
		$controls = document.getElementById('media-manager-controls'),
		$cwd = document.getElementById('media-manager-cwd'),
		$details = document.getElementById('media-manager-details'),
		$placeSelections = $navigator.querySelectorAll('.place-selection'),
		$selectedFile = null,
		dragSource = null,
		dragDropPositions = null,
		dragDropTargets = null,
		uploadQueue = null,
		$fileUploadInput = document.getElementById('media-manager-input-upload'),
		templates = {
			directory: document.getElementById('media-manager-directory-template').innerHTML,
			file: document.getElementById('media-manager-file-template').innerHTML,
			listingHeader: document.getElementById('media-manager-listing-view-header').innerHTML
		},
		$actions = {
			mkdir: $controls.querySelector('[data-action="create-directory"]'),
			upload: $controls.querySelector('[data-action="upload-file"]'),
			touch: $controls.querySelector('[data-action="create-file"]'),
			rename: $controls.querySelector('[data-action="rename"]'),
			rm: $controls.querySelector('[data-action="delete"]')
		},
		detailsEmptyHTML = '<p>' +
			'Select a file to the left to view its details, and double-click to select it.' +
			'</p>',
		files,
		displayedFiles,
		cwd = '',
		view = 'icons',
		self = this,
		db = null,
		ready = false;

	/**
	 * Load settings from local storage or use sane defaults if not.
	 */
	function loadDefaults() {
		let typeLoaded = localStorage.getItem('media-manager-navigator-type'),
			typeAvailable = false;

		// If there's a type loaded, ensure it's in the list of available types for this view.
		$placeSelections.forEach($el => {
			if($el.dataset.type === typeLoaded) {
				typeAvailable = true;
			}
		});

		console.log(typeLoaded, typeAvailable);

		if(typeAvailable) {
			// It's available, try to stick to the last working directory.
			self.setType(
				typeLoaded ||
				$placeSelections.item(0).dataset.type
			);

			self.openDirectory(
				localStorage.getItem('media-manager-navigator-cwd') ||
				''
			);
		}
		else {
			// Not available, set to the first available and a new CWD.
			self.setType($placeSelections.item(0).dataset.type);

			self.openDirectory('');
		}

		self.setView(
			localStorage.getItem('media-manager-navigator-view') ||
			'icons'
		);
	}

	/**
	 * Load the requested Navigator type
	 *
	 * Fetches the baseline of the files for this navigator and calls renders
	 */
	function load() {
		$listing.classList.add('listings-loading');
		$listing.innerHTML = '<p class="message-info">Loading Files...</p>';
		resetPermissions();

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/files',
			{
				parameters: {
					type: type,
					cwd: cwd
				},
				callback: (data, type) => {
					setFiles(data.results);
					render();
					setPermissions(data.permissions);
				}
			}
		)
			.catch(e => {
				$listing.classList.add('listings-loading');
				$listing.innerHTML = '<p class="message-error">Something unexpected happened :(</p>';
				console.log(e);
			});
	}

	/**
	 * Reset the control states to default
	 */
	function resetPermissions() {
		$actions.mkdir.classList.add('disabled');
		$actions.upload.classList.add('disabled');
		$actions.touch.classList.add('disabled');
		$actions.rename.classList.add('disabled');
		$actions.rm.classList.add('disabled');
	}

	function setPermissions(permissions) {
		if(permissions.mkdir) {
			$actions.mkdir.classList.remove('disabled');
		}

		if(permissions.upload) {
			$actions.upload.classList.remove('disabled');
		}

		if(permissions.touch) {
			$actions.touch.classList.remove('disabled');
		}

		if(permissions.rename) {
			$actions.rename.classList.remove('disabled');
		}

		if(permissions.rm) {
			$actions.rm.classList.remove('disabled');
		}
	}
	
	function render() {
		
		if(!ready){
			// Do not attempt to render anything if the application is not marked as ready.
			// This allows set* methods to be used during initiation.
			return;
		}
		
		let html = '', i, $entries;
		
		// The file header gets prepended, (for use on the listing view)
		html += templates.listingHeader;
		
		// Reset this array
		displayedFiles = [];

		if(cwd !== '') {
			// Magic parameter for "up"
			let upDirectory,
				upPath = cwd.split('/');

			upPath.pop();
			upPath.pop();
			upPath = upPath.join('/');
			if(upPath !== '') {
				// End paths with a trailing slash, sans the top-level entry, which should just be empty
				upPath = upPath + '/';
			}

			upDirectory = {
				type: 'directoy',
				name: '↑',
				path: upPath,
				parent: null,
				mtime: null
			};
			displayedFiles.push(upDirectory);
			html += templates.directory.template(upDirectory);
		}

		// Render directories first
		for(i in files){
			if(files[i].parent === cwd && files[i].type === 'directory') {
				displayedFiles.push(files[i]);
				html += templates.directory.template(files[i]);
			}
		}

		// Render files next
		for(i in files){
			if(files[i].parent === cwd && files[i].type !== 'directory') {
				if(matchesMimetype(files[i].type)) {
					displayedFiles.push(files[i]);
					html += templates.file.template(files[i]);
				}
			}
		}

		if(displayedFiles.length === 0) {
			$listing.classList.add('listings-loading');
			html = '<p class="message-info">Directory is empty</p>';
		}
		else {
			$listing.classList.remove('listings-loading');
		}

		$listing.innerHTML = html;
		$details.innerHTML = detailsEmptyHTML;
		$cwd.innerHTML = cwd;
		
		// Some logic needs this.
		$entries = $listing.querySelectorAll('.listing-entry');
		
		// The listings need additional styles for some views.
		if(view === 'icons' || view === 'icons-large') {
			$entries.forEach(el => {
				el.classList.add('tile');
			});
		}

		// Clear out the currently selected file, if any
		$selectedFile = null;
		
		// This can be dynamic if necessary.
		$entries.forEach(el => {
			el.classList.add('selectable');
		});

		renderFilesData();
	}

	function matchesMimetype(type) {
		if($navigator.dataset.mime === '*/*') {
			// Default case, display all files.
			return true;
		}

		let mimes = $navigator.dataset.mime.split(',');

		for(let i = 0; i < mimes.length; i++) {
			if(mimes[i] === type) {
				// Exact match
				return true;
			}
			else if(mimes[i].indexOf('*') !== -1) {
				let pos = mimes[i].indexOf('*');
				if(type.substr(0, pos) === mimes[i].substr(0, pos)) {
					// Partial match
					return true;
				}
			}
		}

		// Nothing matched
		return false;
	}
	
	function renderFilesData() {
		let i, store = getDBFilesStore(), req, count = 0, neededFiles = [];
		
		for(i in displayedFiles){
			req = store.get(displayedFiles[ i ].path);
			
			req.path = displayedFiles[ i ].path;
			req.mtime = displayedFiles[ i ].mtime;
			
			req.onerror = evt => {
				count += 1;
				console.log(evt);
			};
			
			req.onsuccess = evt => {
				count += 1;
				
				if(!evt.target.result) {
					// No result in the local database, look up this file!
					neededFiles.push(evt.target.path);
				}
				else if(evt.target.result.mtime < evt.target.mtime){
					// Too old of a result in the local database.
					neededFiles.push(evt.target.path);
				}
				else{
					// Good result!
					renderFileInfo(evt.target.result);
				}
				
				if(count === displayedFiles.length || neededFiles.length >= 50) {
					// All lookups completed, load any needed file from the server.
					loadFilesData(neededFiles);
					neededFiles = [];
				}
			};
		}
	}

	/**
	 * Render the details of each entry retrieved
	 *
	 * @param file
	 */
	function renderFileInfo(file) {
		let $entry = $listing.querySelector('.listing-entry[data-path="' + file.path + '"]');
		
		if(!$entry){
			return;
		}

		if(view === 'icons') {
			$entry.querySelector('.file-preview img').src = file.preview;
		}
		else if(view === 'icons-large') {
			$entry.querySelector('.file-preview img').src = file.large || file.preview;
		}

		$entry.querySelector('.file-icon img').src = file.icon;
		$entry.querySelector('.file-size').innerHTML = file.size;
		$entry.querySelector('.file-modified').innerHTML = file.mtime_formatted;
		$entry.dataset.icon = file.icon;
		$entry.dataset.preview = file.preview;
		$entry.dataset.size = file.size;
	}
	
	function loadFilesData(neededFiles) {
		if(neededFiles.length === 0) {
			return;
		}
		
		$.ajax({
			url: Core.ROOT_URL + 'mediamanagernavigator/file/info',
			type: 'get',
			data: {
				type: type,
				paths: JSON.stringify(neededFiles)
			},
			dataType: 'json',
			success: function(r){
				var store = getDBFilesStore(true), i, file;
				
				for(i in r.results){
					file = {
						path: r.results[i].path,
						mtime: r.results[i].mtimr,
						mtime_formatted: r.results[i].mtimf,
						size: r.results[i].sizef,
						icon: r.results[i].icon,
						preview: r.results[i].prvw,
						large: r.results[i].large || r.results[i].prvw
					};
					
					store.put(file);
					renderFileInfo(file);
				}
			},
			error: function(r){
				console.log(r);
			}
		});
	}
	
	function startDB() {
		if(db === null){
			console.log('Starting DB Connection');
			window.indexedDB = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
			// DON'T use "var indexedDB = ..." if you're not in a function.
			// Moreover, you may need references to some window.IDB* objects:
			window.IDBTransaction = window.IDBTransaction || window.webkitIDBTransaction || window.msIDBTransaction;
			window.IDBKeyRange = window.IDBKeyRange || window.webkitIDBKeyRange || window.msIDBKeyRange;
			// (Mozilla has never prefixed these objects, so we don't need window.mozIDB*)
			
			var dbOpen = window.indexedDB.open('media-manager-navigator', 3),
				filedbs = ['files_assets', 'files_media', 'files_public', 'files_page'],
				transaction;
			
			dbOpen.onsuccess = function(event) {
				console.log('DB Connection succeeded');
				db = dbOpen.result;
			}
			
			// This event handles the event whereby a new version of the database needs to be created
			// Either one has not been created before, or a new version number has been submitted via the
			// window.indexedDB.open line above
			//it is only implemented in recent browsers
			dbOpen.onupgradeneeded = function(event) {
				console.log('DB Requires upgrade');
				var db = event.target.result, objectStore, i;

				//db.onerror = function(event) {
				//  note.innerHTML += '<li>Error loading database.</li>';
				//};

				if(event.oldVersion < 2) {
					objectStore = db.createObjectStore(
						'files_assets',
						{ keyPath: 'path' }
					);
					objectStore = db.createObjectStore(
						'files_media',
						{ keyPath: 'path' }
					);
					objectStore = db.createObjectStore(
						'files_public',
						{ keyPath: 'path' }
					);
				}

				if(event.oldVersion < 3) {
					objectStore = db.createObjectStore(
						'files_page',
						{ keyPath: 'path' }
					);
				}
			};
		}
	}
	
	function getDBFilesStore(readwrite) {
		
		var rw = readwrite || false,
			mode = rw ? 'readwrite' : 'readonly';
		
		return db.transaction(['files_' + type], mode).objectStore('files_' + type);
	}
	
	function setFiles(updatedfiles) {
		//var store = getDBFilesStore(), i;
		
		files = updatedfiles;
		
		//for(i in files){
		//	store.add(files[ i ]);
		//}
	}

	function processUploadQueue() {
		if(uploadQueue.completedSize >= uploadQueue.totalSize) {
			// When the queue has been completed, drop out and update the results for the user.
			uploadQueue = null;
			load();
			return;
		}

		if(uploadQueue.files.length === 0) {
			// This shouldn't be hit, but is here as a safety check.
			uploadQueue = null;
			load();
			return;
		}

		// FIFO, each iteration of process only deals with one file at a time.
		let nextFile = uploadQueue.files.splice(0, 1)[0],
			xhr = new XMLHttpRequest(),
			chunk,
			$progress = $listing.querySelector('progress'),
			sentSize = 0,
			url;

		url = Core.ROOT_URL + 'mediamanagernavigator/file/upload/chunk' +
			'?type=' + type +
			'&path=' + cwd +
			'&file=' + nextFile.name +
			'&size=' + nextFile.size +
			'&date=' + (new Date().valueOf());

		xhr.onload = evt => {
			let response,
				complete;

			try {
				response = JSON.parse(evt.target.response);
			}
			catch(SyntaxError) {
				response = {status: 400, message: 'Something unexpected happened, please try again later.'}
			}

			if(response.status === 200) {
				// Chunk completed, continue to the next chunk or *NEXT* file if completed.
				sentSize += uploadQueue.chunkSize;
				if(sentSize >= nextFile.size) {
					uploadQueue.completedSize += nextFile.size;
					complete = uploadQueue.completedSize / uploadQueue.totalSize * 100 | 0;
					$progress.value = complete;

					$.notify(nextFile.name + ' uploaded successfully!', 'success');
					processUploadQueue();
				}
				else {
					complete = (uploadQueue.completedSize + sentSize) / uploadQueue.totalSize * 100 | 0;
					$progress.value = complete;

					chunk = nextFile.slice(sentSize, sentSize + uploadQueue.chunkSize);
					xhr.open( 'POST', url );
					xhr.send(chunk);
				}
			}
			else {
				$.notify(response.message, 'error');
				processUploadQueue();
				render();
			}
		};

		xhr.upload.onprogress = evt => {
			// Just update the progress bar.
			if (evt.lengthComputable) {
				let progressCompleted = uploadQueue.completedSize + evt.loaded + sentSize,
					complete = progressCompleted / uploadQueue.totalSize * 100 | 0;

				$progress.value = complete;
			}
		};

		chunk = nextFile.slice(0, uploadQueue.chunkSize)
		xhr.open( 'POST', url );
		xhr.send(chunk);
	}
	
	
	// PUBLIC METHODS

	// Used to differentiate between single and double clicks.
	this.timeout = null;

	this.setType = function(newtype) {
		type = newtype;

		$placeSelections.forEach(el => {
			if(el.dataset.type === type) {
				el.classList.add('active');
			}
			else {
				el.classList.remove('active');
			}
		});

		localStorage.setItem('media-manager-navigator-type', type);
		
		if(ready) {
			// Do not reset the CWD before the app has started; prevents the saved value from being overwritten.
			cwd = '';
			localStorage.setItem('media-manager-navigator-cwd', '');
			load();
		}
	};
	
	this.setView = function(newview) {
		$navigator.classList.remove('view-mode-' + view);
		$navigator.classList.add('view-mode-' + newview);
		
		
		if(newview === 'icons' || newview === 'icons-large') {
			$listing.classList.add('tile-container');
		}
		else {
			$listing.classList.remove('tile-container');
		}
		
		view = newview;

		$controls.querySelectorAll('[data-action="set-view"]').forEach(el => {
			if(el.dataset.view === view) {
				el.classList.add('active');
			}
			else {
				el.classList.remove('active');
			}
		});
		
		if(ready){
			localStorage.setItem('media-manager-navigator-view', view);
			render();
		}
	}
	
	this.openDirectory = function(dir) {
		if(dir === '..'){
			// Special case for "parent" directory.
			var e = cwd.match(/(.*)\/[^\/]*\//);
			dir = e ? e[1] + '/' : '';
		}

		cwd = dir;
		$selectedFile = null;
		
		if(ready){
			localStorage.setItem('media-manager-navigator-cwd', dir);
			resetPermissions();
			render();

			Core.Ajax(
				Core.ROOT_URL + 'mediamanagernavigator/permissions',
				{
					parameters: {
						type: type,
						cwd: cwd
					},
					callback: (data, type) => {
						setPermissions(data.permissions);
					}
				}
			)
				.catch(e => {
					console.log(e);
				});
		}
	};
	
	this.makeDirectory = function(dir) {
		$listing.classList.add('listings-loading');
		$listing.innerHTML = '<p class="message-info">Creating Directory...</p>';

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/directory/mkdir',
			{
				method: 'POST',
				parameters: {
					type: type,
					cwd: cwd,
					path: dir
				},
				callback: (data, responseType) => {
					if(typeof(data.path) !== 'undefined') {
						// Load is required in order for the application to be aware of the changed files.
						cwd = data.path;
						localStorage.setItem('media-manager-navigator-cwd', cwd);
						load();
					}
					else {
						$.notify(data.message, 'error');
						load();
					}
				}
			}
		)
			.catch(e => {
				$.notify('Unable to create directory', 'error');
				load();
				console.log(e);
			});
	};

	this.makeFile = function(name) {
		$listing.classList.add('listings-loading');
		$listing.innerHTML = '<p class="message-info">Creating File...</p>';

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/file/touch',
			{
				method: 'POST',
				parameters: {
					type: type,
					cwd: cwd,
					name: name
				},
				callback: (data, responseType) => {
					if(typeof(data.path) !== 'undefined') {
						// Open the editor for this new file.
						window.location.href = Core.ROOT_URL + 'mediamanagernavigator/file/editor?type=' + type + '&path=' + data.path;
					}
					else {
						$.notify(data.message, 'error');
						load();
					}
				}
			}
		)
			.catch(e => {
				$.notify('Unable to create file', 'error');
				load();
				console.log(e);
			});
	};

	this.renameFile = function(name) {
		$listing.classList.add('listings-loading');
		$listing.innerHTML = '<p class="message-info">Renaming File...</p>';

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/file/rename',
			{
				method: 'POST',
				parameters: {
					type: type,
					cwd: cwd,
					path: $selectedFile.dataset.path,
					name: name
				},
				callback: (data, responseType) => {
					load();
				}
			}
		)
		.catch(e => {
			$.notify('Unable to rename file', 'error');
			load();
			console.log(e);
		});
	};
	
	this.moveFile = function(path) {
		$listing.classList.add('listings-loading');
		$listing.innerHTML = '<p class="message-info">Moving File...</p>';

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/file/move',
			{
				method: 'POST',
				parameters: {
					type: type,
					path: $selectedFile.dataset.path,
					dest: path
				},
				callback: (data, responseType) => {
					load();
				}
			}
		)
			.catch(e => {
				$.notify('Unable to move file', 'error');
				load();
				console.log(e);
			});
	};

	this.uploadFiles = function(files) {
		let file, $progress, formData = new FormData(), xhr = new XMLHttpRequest(), fname, i,
			totalSize = 0,
			chunkSize = parseInt($navigator.dataset.maxUpload),
			perFileSent = 0,
			chunk,
			completedSize = 0;
		
		if(files.length === 1) {
			fname = files[0].name.replace(/[^a-zA-Z0-9\.-]*/g, '');
		}
		else {
			fname = files.length + ' Files';
		}

		$listing.classList.add('listings-loading');
		$listing.innerHTML = '<p class="message-info">Uploading ' + fname + '...</p><progress max="100" value="0"></progress>';

		$progress = $listing.querySelector('progress');


		uploadQueue = {
			totalSize: 0,
			completedSize: 0,
			chunkSize: chunkSize,
			files: [],
		};

		// Build the total size of files to upload
		for(i = 0; i < files.length; i++) {
			uploadQueue.totalSize += files[i].size;
			uploadQueue.files.push(files[i]);
		}


		// Now that the queue has been generated, kick off the actual process handler
		// This is the method that will actually step through each file and perform the POST
		processUploadQueue();
	}
	
	this.selectEntry = function($element) {
		// Deselect the previous element, (if any)
		if($selectedFile) {
			$selectedFile.classList.remove('active');
		}

		resetPermissions();

		// Set the currently selected file for use by the file controls
		$selectedFile = $element;

		$selectedFile.classList.add('active');
		$details.innerHTML = 'Loading Details...';

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/file/details',
			{
				parameters: {
					type: type,
					cwd: cwd,
					path: $selectedFile.dataset.path
				},
				callback: (data, type) => {
					$details.innerHTML = data.body;
					setPermissions(data.permissions);

					Object.keys(data.data).forEach( key => {
						$selectedFile.dataset[ key ] = data.data[ key ];
					});
				}
			}
		)
			.catch(e => {
				$details.innerHTML = '<p class="message-error">Unable to load details for entry</p>';
				console.log(e);
			});
	};
	
	this.openFile = function($element) {
		// By default, just open this file.
		window.location.href = Core.ROOT_URL + 'mediamanagernavigator/file/editor?type=' + type + '&path=' + $element.dataset.path;
	};

	/**
	 * Delete a file or directory from the listing
	 *
	 * @param $element
	 */
	this.deleteSelectedEntry = function() {
		$details.innerHTML = 'Deleting...';

		Core.Ajax(
			Core.ROOT_URL + 'mediamanagernavigator/file/delete?type=' + type,
			{
				method: 'POST',
				parameters: {
					type: type,
					path: $selectedFile.dataset.path
				},
				callback: (data, type) => {
					load();
				}
			}
		)
			.catch(e => {
				$details.innerHTML = '<p class="message-error">Unable to delete entry!</p>';
				console.log(e);
			});
	}
	
	
	// EVENTS!
	$placeSelections.forEach(el => {
		el.addEventListener('click', evt => {
			let btn = evt.target.closest('a');

			this.setType(btn.dataset.type);
			evt.preventDefault();
		});
	});

	$controls.querySelectorAll('.run-action').forEach(el => {
		el.addEventListener('click', evt => {
			let btn = evt.target.closest('a'),
				action = btn.dataset.action,
				userInput;

			evt.preventDefault();

			if(btn.classList.contains('disabled')) {
				return;
			}

			switch(action) {
				case 'set-view':
					this.setView(btn.dataset.view);
					break;
				case 'go-up':
					this.openDirectory('..');
					break;
				case 'delete':
					if(confirm('Confirm deleting the selected entry?')) {
						this.deleteSelectedEntry();
					}
					break;
				case 'create-directory':
					userInput = prompt('New directory name');

					if(userInput !== null && userInput.trim() !== ''){
						this.makeDirectory(userInput);
					}
					break;
				case 'create-file':
					userInput = prompt('New file name');

					if(userInput !== null && userInput.trim() !== ''){
						this.makeFile(userInput);
					}
					break;
				case 'upload-file':
					$fileUploadInput.click();
					break;
				case 'rename':
					let newName = $selectedFile.dataset.path,
						slashPos = newName.lastIndexOf('/'),
						dotPos,
						extension = '';

					if(slashPos !== -1) {
						newName = newName.substr(slashPos + 1);
					}

					dotPos = newName.indexOf('.');
					if(dotPos !== -1) {
						extension = newName.substr(dotPos + 1);
						newName = newName.substr(0, dotPos);
					}

					newName = prompt('Please enter a new filename', newName);

					if(newName !== null && newName !== '') {
						// Reapply the extension
						if(extension !== '') {
							newName = newName + '.' + extension;
						}

						this.renameFile(newName);
					}
					break;
			}
		});
	});

	$fileUploadInput.addEventListener('change', evt => {
		this.uploadFiles(evt.target.files);
	});

	$listing.addEventListener('click', evt => {
		let match;

		if((match = evt.target.closest('.listing-entry'))) {
			if(this.timeout) {
				clearTimeout(this.timeout);
			}

			// Toggle the UI immediately, this helps with user perception and does not affect the application state
			if($selectedFile) {
				$selectedFile.classList.remove('active');
			}
			match.classList.add('active');

			this.timeout = setTimeout(() => { this.selectEntry(match); }, 250);
			evt.preventDefault();
		}
	});

	$listing.addEventListener('dblclick', evt => {
		let match;

		if((match = evt.target.closest('.file'))) {
			if(this.timeout) {
				clearTimeout(this.timeout);
			}

			this.openFile(match);
			evt.preventDefault();
		}
		else if((match = evt.target.closest('.directory'))) {
			if(this.timeout) {
				clearTimeout(this.timeout);
			}

			this.openDirectory(match.dataset.path);
			evt.preventDefault();
		}
	});

	this.dragEventHandler = evt => {
		if(dragSource) {
			let absoluteX = evt.clientX,
				absoluteY = evt.clientY,
				pos, el;

			dragSource.style.left = absoluteX + 'px';
			dragSource.style.top = absoluteY + 'px';

			for(let i = 0; i < dragDropTargets.length; i++) {
				el = dragDropTargets[i];

				pos = el.getBoundingClientRect();

				if(
					pos.left < absoluteX &&
					pos.right > absoluteX &&
					pos.top < absoluteY &&
					pos.bottom > absoluteY
				) {
					el.classList.add('drag-over');
				}
				else {
					el.classList.remove('drag-over');
				}
			}

			evt.preventDefault();
		}
	}

	this.mouseUpHandler = evt => {
		if(dragSource) {
			$listing.querySelectorAll('.directory.drag-over').forEach(el => {

				this.moveFile(el.dataset.path);
				el.classList.remove('drag-over');
			});

			document.body.removeChild(dragSource);
			dragSource = null;
			dragDropTargets = null;
		}
	}

	$listing.addEventListener('mousedown', evt => {
		let match;

		if(evt.buttons === 1 && (match = evt.target.closest('.file'))) {
			if(this.timeout) {
				clearTimeout(this.timeout);
			}

			this.timeout = setTimeout(() => {
				this.selectEntry(match);
				dragSource = match.cloneNode(true);
				dragSource.classList.add('media-manager-drag-proxy');
				dragSource.style.left = (evt.clientX) + 'px';
				dragSource.style.top = (evt.clientY) + 'px';
				document.body.appendChild(dragSource);

				dragSource.addEventListener('mousemove', this.dragEventHandler);
				dragSource.addEventListener('mouseup', this.mouseUpHandler);

				dragDropTargets = [];
				$listing.querySelectorAll('.directory').forEach(el => {
					dragDropTargets.push(el);
				});
			}, 230);

			evt.preventDefault();
		}
	});

	$listing.addEventListener('mousemove', this.dragEventHandler);

	$listing.addEventListener('mouseup', this.mouseUpHandler);

	$listing.addEventListener('dragover', evt => {
		evt.preventDefault();
	});

	$listing.addEventListener('dragend', evt => {
		evt.preventDefault();
	});

	$listing.addEventListener('drop', evt => {
		evt.preventDefault();

		console.log(evt);

		//this.uploadFiles(evt.dataTransfer.files);
	});
	
	
	loadDefaults();

	startDB();
	
	
	// Load everything at the start.
	ready = true;
	load();
};
