"use strict";

class NavigationManager2 {
	constructor(element) {
		this.element = element;
		this.counter = 0;
		this.list = element.querySelector('.navigation-edit-entries');

		if(this.element.querySelector('.open-menu-widgetareas')) {
			this.element.querySelector('.open-menu-widgetareas').addEventListener(
				'click',
				evt => {
					this.buildWidgetAreaDialog();
					evt.preventDefault();
				}
			);
		}
		
		this.element.querySelector('.add-entry-int-btn').addEventListener(
			'click',
			evt => {
				this.addInternalLinkEvent();
				evt.preventDefault();
			}
		);

		this.element.querySelector('.add-entry-ext-btn').addEventListener(
			'click',
			evt => {
				this.addExternalLinkEvent();
				evt.preventDefault();
			}
		);

		this.element.querySelector('.add-entry-none-btn').addEventListener(
			'click',
			evt => {
				this.addTextLinkEvent();
				evt.preventDefault();
			}
		);

		this.element.querySelector('.save-menu').addEventListener(
			'click',
			evt => {
				this.saveMenu();
				evt.preventDefault();
			}
		);

		this.list.addEventListener(
			'click',
			evt => {
				if(evt.target.closest('.edit-entry-link')) {
					evt.preventDefault();

					let li = evt.target.closest('li');
					if(li.dataset.type === 'int') {
						this.editInternalLinkEvent(li);
					}
					else if(li.dataset.type === 'ext') {
						this.editExternalLinkEvent(li);
					}
					else {
						$.notify('Unsupported link type, unable to open editor!', 'error');
					}
				}
				else if(evt.target.closest('.delete-entry-link')) {
					evt.preventDefault();

					let li = evt.target.closest('li');
					this.deleteLink(li);
				}
				else if(evt.target.closest('.undo-delete-entry-link')) {
					evt.preventDefault();

					let li = evt.target.closest('li');
					this.undeleteLink(li);
				}
			}
		);

		if(this.list.querySelector('li')) {
			// There are existing entries on this menu; load them into the interface.
			this.loadLinks();
		}

		$(this.list).nestedSortable({
			handle:              '.entry-title-label',
			items:               'li',
			forcePlaceholderSize: true,
			helper:               'clone',
			opacity:              .6,
			placeholder:          'placeholder',
			tabSize:              25,
			maxLevels:            2,
			tolerance:            'pointer'
		});

		if(window.location.hash === ('#menu:' + this.element.dataset.id.toLowerCase())) {
			this.element.classList.remove('collapsed');
			this.element.classList.add('expanded');
		}
	}

	buildInternalLinkDialog() {
		let dialog = new Core.ModalDialog(),
			pageSelect,
			titleInput,
			submitButton;

		dialog.open();
		dialog.frame.innerHTML = document.querySelector('.add-entry-options-int').innerHTML;

		pageSelect = dialog.frame.querySelector('select[name=url]');
		titleInput = dialog.frame.querySelector('input[name=title]');
		submitButton = dialog.frame.querySelector('.submit-btn');

		pageSelect.addEventListener('change', evt => {
			let pageTitle = pageSelect.options[ pageSelect.selectedIndex ].text;

			while(pageTitle.indexOf('»') !== -1) {
				pageTitle = pageTitle.substr(pageTitle.indexOf('»') + 2);
			}

			pageTitle = pageTitle.replace(/ \(.*\)/, '');

			titleInput.value = pageTitle;
		});

		submitButton.addEventListener('click', evt => {
			evt.preventDefault();

			if(pageSelect.value === '') {
				$.notify('Please select a page', 'error');
				return;
			}

			if(titleInput.value === '') {
				$.notify('Please enter a label', 'error');
				return;
			}

			this.saveInternalLinkFromFrame(dialog.frame);
			dialog.close();
		});

		titleInput.addEventListener('keyup', evt => {
			if(evt.key === 'Enter') {
				submitButton.click();
			}
		});

		return dialog.frame;
	}

	addInternalLinkEvent() {
		let frame = this.buildInternalLinkDialog();

		frame.querySelector('input[name=id]').value = 'NEW';
	}

	editInternalLinkEvent(li) {
		let frame = this.buildInternalLinkDialog();

		frame.querySelector('input[name=id]').value = li.dataset.id;
		frame.querySelector('select[name=url]').value = li.dataset.url;
		frame.querySelector('input[name=title]').value = li.dataset.title;
		frame.querySelector('select[name=target]').value = li.dataset.target;
	}

	saveInternalLinkFromFrame(frame) {
		let pageSelect = frame.querySelector('select[name=url]'),
			titleInput = frame.querySelector('input[name=title]'),
			targetSelect = frame.querySelector('select[name=target]'),
			idInput = frame.querySelector('input[name=id]'),
			data = {
				type: 'int',
				target: targetSelect.value,
				id: idInput.value,
				url: pageSelect.value,
				title: Core.Strings.trim(titleInput.value).replace(/"/g, ''),
				parent: 0,
				status: 'new'
			};

		this.saveLink(data);
	}

	buildExternalLinkDialog() {
		let dialog = new Core.ModalDialog(),
			pageInput,
			titleInput,
			submitButton;

		dialog.open();
		dialog.frame.innerHTML = document.querySelector('.add-entry-options-ext').innerHTML;

		pageInput = dialog.frame.querySelector('input[name=url]');
		titleInput = dialog.frame.querySelector('input[name=title]');
		submitButton = dialog.frame.querySelector('.submit-btn');

		pageInput.addEventListener('blur', evt => {
			if(evt.target.value.indexOf('://') === -1) {
				evt.target.value = 'https://' + evt.target.value;
			}
		});

		submitButton.addEventListener('click', evt => {
			evt.preventDefault();

			if(pageInput.value === '') {
				$.notify('Please enter a URL', 'error');
				return;
			}

			if(pageInput.value.indexOf('://') === -1) {
				$.notify('Please ensure you enter a valid URL', 'error');
				return;
			}

			if(titleInput.value === '') {
				$.notify('Please enter a label', 'error');
				return;
			}

			this.saveExternalLinkFromFrame(dialog.frame);
			dialog.close();
		});

		pageInput.addEventListener('keyup', evt => {
			if(evt.key === 'Enter') {
				titleInput.focus();
			}
		});

		titleInput.addEventListener('keyup', evt => {
			if(evt.key === 'Enter') {
				submitButton.click();
			}
		});

		return dialog.frame;
	}

	addExternalLinkEvent() {
		let frame = this.buildExternalLinkDialog();

		frame.querySelector('input[name=id]').value = 'NEW';
	}

	editExternalLinkEvent(li) {
		let frame = this.buildExternalLinkDialog();

		frame.querySelector('input[name=id]').value = li.dataset.id;
		frame.querySelector('input[name=url]').value = li.dataset.url;
		frame.querySelector('input[name=title]').value = li.dataset.title;
		frame.querySelector('select[name=target]').value = li.dataset.target;
	}

	saveExternalLinkFromFrame(frame) {
		let pageInput = frame.querySelector('input[name=url]'),
			titleInput = frame.querySelector('input[name=title]'),
			targetSelect = frame.querySelector('select[name=target]'),
			idInput = frame.querySelector('input[name=id]'),
			data = {
				type: 'ext',
				target: targetSelect.value,
				id: idInput.value,
				url: Core.Strings.trim(pageInput.value),
				title: Core.Strings.trim(titleInput.value).replace(/"/g, ''),
				parent: 0,
				status: 'new'
			};

		this.saveLink(data);
	}

	buildTextLinkDialog() {
		let dialog = new Core.ModalDialog(),
			titleInput,
			submitButton;

		dialog.open();
		dialog.frame.innerHTML = document.querySelector('.add-entry-options-none').innerHTML;

		titleInput = dialog.frame.querySelector('input[name=title]');
		submitButton = dialog.frame.querySelector('.submit-btn');

		submitButton.addEventListener('click', evt => {
			evt.preventDefault();

			if(titleInput.value === '') {
				$.notify('Please enter a label', 'error');
				return;
			}

			this.saveTextLinkFromFrame(dialog.frame);
			dialog.close();
		});

		titleInput.addEventListener('keyup', evt => {
			if(evt.key === 'Enter') {
				submitButton.click();
			}
		});

		return dialog.frame;
	}

	addTextLinkEvent() {
		let frame = this.buildTextLinkDialog();

		frame.querySelector('input[name=id]').value = 'NEW';
	}

	editTextLinkEvent(li) {
		let frame = this.buildTextLinkDialog();

		frame.querySelector('input[name=id]').value = li.dataset.id;
		frame.querySelector('input[name=title]').value = li.dataset.title;
	}

	saveTextLinkFromFrame(frame) {
		let titleInput = frame.querySelector('input[name=title]'),
			idInput = frame.querySelector('input[name=id]'),
			data = {
				type: 'none',
				target: '',
				id: idInput.value,
				url: '',
				title: Core.Strings.trim(titleInput.value).replace(/"/g, ''),
				parent: 0,
				status: 'new'
			};

		this.saveLink(data);
	}

	buildWidgetAreaDialog() {
		let dialog = new Core.ModalDialog();

		dialog.open();
		dialog.frame.innerHTML = this.element.querySelector('.widgetarea-template').innerHTML;

		dialog.frame.querySelectorAll('.install-menu').forEach(link => {
			link.addEventListener('click', evt => {
				evt.preventDefault();

				let button = evt.target.closest('.button'),
					data = {
						template: button.dataset.template,
						baseurl: button.dataset.pageBaseurl,
						widget_baseurl: '/navigation/view/' + button.dataset.id,
						area: button.dataset.widgetarea
					};

				button.classList.add('loading');

				Core.Ajax(
					Core.ROOT_URL + 'widget/instance/install',
					{
						data: data,
						dataType: 'form',
						method: 'POST',
						callback: (response, ctype) => {
							if(response.status === 200) {
								window.location.href = response.location;
							}
							else {
								$.notify('Unable to install widget', 'error');
								button.classList.remove('loading');
							}
						}
					}
				);
			});
		});

		dialog.frame.querySelectorAll('.remove-menu').forEach(link => {
			link.addEventListener('click', evt => {
				evt.preventDefault();

				let button = evt.target.closest('.button');

				button.classList.add('loading');

				Core.Ajax(
					button.href,
					{
						method: 'POST',
						callback: (response, ctype) => {
							if(response.status === 200) {
								Core.Reload();
							}
							else {
								$.notify('Unable to install widget', 'error');
								button.classList.remove('loading');
							}
						}
					}
				);
			});
		});

		return dialog.frame;
	}

	loadLinks() {
		let data = [];

		this.list.querySelectorAll('li').forEach(el => {
			let linkData = {
				type: el.dataset.type,
				target: el.dataset.target,
				id: el.dataset.id,
				url: el.dataset.url,
				title: el.dataset.title,
				parent: el.dataset.parent,
				status: el.dataset.status
			};

			data.push(linkData);
		});

		// Clear the interface; we'll be building new.
		this.list.innerHTML = '';

		for(let i = 0; i < data.length; i++) {
			this.attachLinkToGUI(data[i]);
		}
	}

	saveLink(data) {
		let isNew = (data.id === 'NEW');

		if(isNew) {
			data.id = 'new-' + (this.counter++);
		}

		this.attachLinkToGUI(data);
	}

	deleteLink(li) {
		// FIRST, delete any children links.
		li.querySelectorAll('li').forEach(subLink => {
			this.deleteLink(subLink);
		});

		if(li.dataset.status === 'new') {
			li.dataset.status = 'new-deleted';
		}
		else if(li.dataset.status === 'existing') {
			li.dataset.status = 'deleted';
		}

		li.classList.add('deleted-link');
	}

	undeleteLink(li) {
		// First, undelete any parents
		let parentLink = li.parentElement.closest('li');
		if(parentLink) {
			if(parentLink.dataset.status === 'deleted' || parentLink.dataset.status === 'new-deleted') {
				this.undeleteLink(parentLink);
			}
		}

		if(li.dataset.status === 'new-deleted') {
			li.dataset.status = 'new';
		}
		else {
			li.dataset.status = 'existing';
		}

		li.classList.remove('deleted-link');
	}

	attachLinkToGUI(data) {
		let htmlContent = document.getElementById('navigation-entry-template').innerHTML,
			parentLink = null,
			attachTarget;

		htmlContent = Core.Strings.template(htmlContent, data);

		if(data.parent) {
			// Look for this parent entry
			parentLink = this.list.querySelector('li[data-id="' + data.parent + '"]');

			if(parentLink) {
				// Ensure it has a sublist
				if(!parentLink.querySelector('ol')) {
					attachTarget = document.createElement('ol');
					parentLink.append(attachTarget);
				}
				else {
					attachTarget = parentLink.querySelector('ol');
				}
			}
			else {
				attachTarget = this.list;
			}
		}
		else {
			attachTarget = this.list;
		}


		if(attachTarget.querySelector('li[data-id="' + data.id + '"]')) {
			let tempNode = document.createElement('ol'),
				targetNode = attachTarget.querySelector('li[data-id="' + data.id + '"]');

			tempNode.innerHTML = htmlContent;
			targetNode.parentNode.replaceChild(tempNode.firstElementChild, targetNode);

			tempNode.remove();
		}
		else {
			attachTarget.innerHTML += htmlContent;
		}
	}
	
	saveMenu() {
		let saveData = {
			name: null,
			id: null,
			entries: []
		};

		this.element.querySelector('.save-menu').classList.add('loading');
		$.notify('Saving menu, please wait...', 'info');

		// Delete any children links that may have been moved to under a parent of a deleted link.
		this.list.querySelectorAll('li').forEach(el => {
			if(el.dataset.status === 'deleted' || el.dataset.status === 'new-deleted') {
				// This function is safe to call multiple times and will handle recursion.
				this.deleteLink(el);
			}
		});

		this.list.querySelectorAll('li').forEach(el => {
			let parentEntry = el.parentElement.closest('li'),
				data = {};

			if(parentEntry) {
				data.parent = parentEntry.dataset.id;
			}
			else {
				data.parent = '';
			}

			data.id = el.dataset.id;
			data.type = el.dataset.type;
			data.url = el.dataset.url;
			data.target = el.dataset.target;
			data.title = el.dataset.title;
			data.status = el.dataset.status;

			if(el.dataset.status !== 'new-deleted') {
				// New links that have been deleted can just be skipped entirely.
				saveData.entries.push(data);
			}
		});

		saveData.name = this.element.querySelector('.menu-name-input').value;
		saveData.id = this.element.querySelector('.menu-id-input').value;

		Core.Ajax(
			Core.ROOT_URL + 'navigation/savemenu',
			{
				method: 'POST',
				data: saveData,
				callback: (data) => {
					if(data.location) {
						// Set the new hash, then call reload.
						// This is required because otherwise the base URL doesn't actually change.
						window.location.href = data.location;
					}
					Core.Reload();
				}
			}
		)
			.catch(e => {
				this.element.querySelector('.save-menu').classList.remove('loading');
				$.notify(e.message, error);
			});
	}
}