<?php

use Core\Model;
use Minishlink\WebPush\Subscription;

class PwaUserSubscriptionModel extends Model {
	public static $Schema = [
		'id'                   => [
			'type'     => Model::ATT_TYPE_UUID,
			'required' => true,
			'null'     => false,
		],
		'user' => [
			'type' => Model::ATT_TYPE_UUID_FK,
			'comment' => 'Associated user ID',
			'required' => false,
			'null' => false,
			'link' => [
				'model' => 'User',
				'type' => Model::LINK_BELONGSTOONE,
				'on' => 'id',
			],
		],
		'useragent' => [
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 128,
			'formatter' => 'Core\\Formatter\\GeneralFormatter::UserAgent',
		],
		'endpoint' => [
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 256,
		],
		'encoding' => [
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 16,
		],
		'auth' => [
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 32,
		],
		'public_key' => [
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 96,
		]
	];

	public static $Indexes = [
		'primary'   => ['id'],
		'user'      => ['user'],
		'user_auth' => ['user', 'auth'],
	];


	public static $HasCreated = true;
	public static $HasUpdated = true;

	public function getAsSubscription() : Subscription {
		return Subscription::create([
			'endpoint' => $this->endpoint,
			'publicKey' => $this->public_key,
			'authToken' => $this->auth,
			'contentEncoding' => $this->encoding
		]);
	}
}