/**
 * Replicates console.*, with only running when in DEV MODE.
 *
 * @type {{warn: dev.warn, log: dev.log, error: dev.error, info: dev.info}}
 */
dev = {
	log: function(msg) {
		if(DEV_MODE) {
			console.log(msg);
		}
	},
	info: function(msg) {
		if(DEV_MODE) {
			console.info(msg);
		}
	},
	warn: function(msg) {
		if(DEV_MODE) {
			console.warn(msg);
		}
	},
	error: function(msg) {
		if(DEV_MODE) {
			console.error(msg);
		}
	}
};


function fetchFromCache(request, cachename) {
	return caches
		.open(cachename)
		.then(
			function(cache) {
				return cache
					.match(request)
					.then(
						function(match) {
							if(typeof(match) === 'undefined') {
								// Not located, use the network to fetch instead.
								//dev.log('Not cached, pulling from network: ' + request.url);
								return fetchFromNetworkAndCache(request, cache);
							}
							else {
								//dev.log('Returning cached response: ' + request.url);
								return match;
							}
						},
						function() {
							//dev.error('Unable to lookup request in cache: ' + request.url);
							return Promise.reject('Unable to lookup request in cache');
						}
					);
			},
			function() {
				//dev.error('Unable to open client cache!');
				return Promise.reject('Unable to open client cache');
			}
		);
}

/**
 * Fetch a resource from the network and cache the result.
 *
 * @param request
 * @param cache
 * @returns {Promise<IDBRequest<IDBValidKey> | void | never>}
 */
function fetchFromNetworkAndCache(request, cache) {
	return fetch(request)
		.then(
			function(response) {
				return cache.put(request, response.clone()).then(function() {
					//dev.log('Retrieved response and cached: ' + request.url);
					return response;
				});
			}
		);
}