<p class="message-info" id="push-notification-message">
	Loading...
</p>

<div class="button-group">
	<a href="#" class="button" id="push-notification-toggle" disabled="disabled">
		Loading...
	</a>
	<a href="#" class="button" id="push-notification-test" disabled="disabled">
		<i class="icon icon-archive"></i>
		<span>Test Notifications</span>
	</a>
</div>


{script library="notify"}{/script}
{script library="jqueryui"}{/script}
{script location="foot"}<script>
	// Based on code from https://github.com/Minishlink/web-push-php-example/blob/master/src/app.js

	$(function(){
		var supported = null,
			$message = $('#push-notification-message'),
			$button = $('#push-notification-toggle'),
			$testbutton = $('#push-notification-test'),
			subscribed = false;

		const serverkey = '{$public_key}',
			i18n = {
				message: {
					incompatible_browser: 'Please update your browser to make use of notifications!',
					not_enrolled: 'You are not currently receiving notifications.',
					disabled: 'You have requested not to receive notifications.  If you wish to receive them, please allow the "Notification" permission in your browser.',
					network_error: 'There was a network error while trying to update your preferences, please try again later.',
					requesting: 'Requesting permissions, please allow the "Notification" permission when prompted.',
					enrolled: 'You are enrolled to receive notifications!',
					test_submitted: 'You should see a test notification in a moment!',
					test_error: 'Unable to send a test notification, please reload the page or try resubscribing.'
				}
			};

		function urlBase64ToUint8Array(base64String) {
			const padding = '='.repeat((4 - (base64String.length % 4)) % 4);
			const base64 = (base64String + padding).replace(/\-/g, '+').replace(/_/g, '/');

			const rawData = window.atob(base64);
			const outputArray = new Uint8Array(rawData.length);

			for (let i = 0; i < rawData.length; ++i) {
				outputArray[i] = rawData.charCodeAt(i);
			}
			return outputArray;
		}

		function pushNotificationStateChange(state, message) {
			switch(state) {
				case 'incompatible':
					supported = false;
					subscribed = false;
					$message.switchClass('message-info message-success', 'message-error').html(message);
					$button.hide();
					$testbutton.hide();
					break;

				case 'default':
					supported = true;
					subscribed = false;
					$message.switchClass('message-error message-success', 'message-info').html(message);
					$button.removeAttr('disabled').html('Enable Notifications');
					$testbutton.attr('disabled', 'disabled');
					break;

				case 'requesting':
					$message.html(message);
					$button.attr('disabled', 'disabled').html('Requesting Permission...');
					break;

				case 'denied':
					$message.html(message);
					subscribed = false;
					$button.removeAttr('disabled').html('Enable Notifications');
					break;

				case 'enrolled':
					$message.switchClass('message-info message-error', 'message-success').html(message);
					subscribed = true;
					$button.removeAttr('disabled').html('Disable Notifications');
					$testbutton.removeAttr('disabled');
					break;
			}
		}

		function checkNotificationPermission() {
			return new Promise((resolve, reject) => {
				if (Notification.permission === 'denied') {
					return reject(new Error('Push messages are blocked.'));
				}

				if (Notification.permission === 'granted') {
					return resolve();
				}

				if (Notification.permission === 'default') {
					return Notification.requestPermission().then(result => {
						if (result !== 'granted') {
							reject(new Error('Bad permission result'));
						}

						resolve();
					});
				}
			});
		}

		/**
		 * Run the necessary checks for support on the features needed.
		 */
		function checkSupport() {
			if (!('serviceWorker' in navigator)) {
				console.warn('Service workers are not supported by this browser');
				pushNotificationStateChange('incompatible', i18n.message.incompatible_browser);
				return;
			}

			if (!('PushManager' in window)) {
				console.warn('Push notifications are not supported by this browser');
				pushNotificationStateChange('incompatible', i18n.message.incompatible_browser);
				return;
			}

			if (!('showNotification' in ServiceWorkerRegistration.prototype)) {
				console.warn('Notifications are not supported by this browser');
				pushNotificationStateChange('incompatible', i18n.message.incompatible_browser);
				return;
			}

			if(Notification.permission === 'default') {
				pushNotificationStateChange('default', i18n.message.not_enrolled);
				return;
			}

			if(Notification.permission === 'denied') {
				pushNotificationStateChange('denied', i18n.message.disabled);
				return;
			}

			if(Notification.permission === 'granted') {
				// Check and see if this user has a subscription set.
				navigator.serviceWorker.ready
					.then(serverWorkerRegistration => {
						return serverWorkerRegistration.pushManager.getSubscription()
					})
					.then(subscription => {
						if(subscription) {
							pushNotificationStateChange('enrolled', i18n.message.enrolled);

							return pushSubscription(subscription, 'PUT');
						}
						else {
							pushNotificationStateChange('default', i18n.message.not_enrolled);
						}
					});
			}
		}

		$button.click(function() {
			if($(this).attr('disabled') === 'disabled') {
				return false;
			}

			if(!subscribed) {
				pushNotificationStateChange('requesting', i18n.message.requesting);

				checkNotificationPermission()
					// Wait for the service worker to become ready
					.then(() => navigator.serviceWorker.ready)
					// Register this device and the connected server
					.then(serverWorkerRegistration => {
						return serverWorkerRegistration.pushManager.subscribe({
							userVisibleOnly: true,
							applicationServerKey: urlBase64ToUint8Array(serverkey)
						});
					})
					// Notify the connected server of this device's credentials
					.then(subscription => {
						return pushSubscription(subscription, 'POST');
					})
					.then(subscription => subscription && pushNotificationStateChange('enrolled', i18n.message.enrolled))
					.catch(e => {
						if (Notification.permission === 'denied') {
							// The user denied the notification permission which
							// means we failed to subscribe and the user will need
							// to manually change the notification permission to
							// subscribe to push messages
							console.warn('Notifications are denied by the user.');
							pushNotificationStateChange('denied', i18n.message.disabled);
						} else {
							// A problem occurred with the subscription; common reasons
							// include network errors or the user skipped the permission
							console.error('Impossible to subscribe to push notifications', e);
							pushNotificationStateChange('incompatible', i18n.message.network_error);
						}
					});
			}
			else {
				navigator.serviceWorker.ready
					.then(sw => {
						return sw.pushManager.getSubscription();
					})
					.then(sub => {
						return pushSubscription(sub, 'DELETE');
					})
					.then(sub => {
						return sub.unsubscribe();
					})
					.then(() => pushNotificationStateChange('default', i18n.message.not_enrolled))
					.catch(e => {
						$.notify(e.message, 'error');
					});
			}

			return false;
		});

		$testbutton.click(function() {
			if($(this).attr('disabled') === 'disabled') {
				return false;
			}

			navigator.serviceWorker.ready
				.then(sw => {
					return sw.pushManager.getSubscription()
				})
				.then(sub => {
					if(sub) {
						let auth = sub.getKey('auth');
						auth = btoa(String.fromCharCode.apply(null, new Uint8Array(auth)));

						return fetch(Core.ROOT_URL + 'pwa/notifications/test?auth=' + auth, { method: 'PUT' });
					}
					else {
						reject(i18n.message.test_error);
					}
				})
				.then(response => {
					return response.json();
				})
				.then(data => {
					$.notify(data.message, data.status);
				})
				.catch(e => {
					$.notify(e.message, 'error');
				});
		});

		function pushSubscription(subscription, method) {
			const key = subscription.getKey('p256dh');
			const token = subscription.getKey('auth');
			const contentEncoding = (PushManager.supportedContentEncodings || ['aesgcm'])[0];

			return fetch(Core.ROOT_URL + 'pwa/notifications/subscribe', {
				method,
				body: JSON.stringify({
					endpoint: subscription.endpoint,
					publicKey: key ? btoa(String.fromCharCode.apply(null, new Uint8Array(key))) : null,
					authToken: token ? btoa(String.fromCharCode.apply(null, new Uint8Array(token))) : null,
					contentEncoding,
				}),
			}).then(() => subscription);
		}

		checkSupport();
	});
</script>{/script}