<?php


namespace SecuritySuite;


use Core\ConfigHandler;
use Core\Datamodel\Dataset;
use Core\Date\DateTime;

class LoginHandler {
	/**
	 * Hook into /user/login/failed
	 *
	 * @param null|\UserModel $user
	 */
	public static function FailedLoginHook($user = null) {

		$since = DateTime::NowGMT() - SECONDS_ONE_HOUR;
		$configFailedLoginsForgotPassword = ConfigHandler::Get('/security/user/failed_logins_forgot_password_reminder');
		$configFailedLoginsSlowdownCount = ConfigHandler::Get('/security/user/failed_logins_slowdown_count');
		$configFailedLoginsSlowdownDelay = ConfigHandler::Get('/security/user/failed_logins_slowdown_delay');
		$configFailedLoginsBlockCount = ConfigHandler::Get('/security/user/failed_logins_block_count');
		$configFailedLoginsBlockDuration = ConfigHandler::Get('/security/user/failed_logins_block_duration');
		$configFailedLoginsRandomSleep = ConfigHandler::Get('/security/user/failed_logins_random_delay');

		$failedLogins = Dataset::Init()
			->count()
			->table('system_log')
			->whereEquals('code', '/user/login')
			->whereEquals('level', 'high')
			->whereEquals('ip_addr', REMOTE_IP)
			->where('datetime > ' . $since)
			->executeAndGet();

		// @todo Make use of this and $configFailedLoginsForgotPassword
		// to send the user with a password reminder automatically.
		if($user && $user instanceof \UserModel) {
			$failedUserLogins = Dataset::Init()
				->count()
				->table('system_log')
				->whereEquals('code', '/user/login')
				->whereEquals('level', 'high')
				->whereEquals('affected_user_id', $user->id)
				->where('datetime > ' . $since)
				->executeAndGet();
		}
		else {
			$failedUserLogins = 0;
		}

		if($configFailedLoginsBlockCount && $failedLogins >= $configFailedLoginsBlockCount) {
			IpBlacklistHelper::BlockIP(
				REMOTE_IP,
				$configFailedLoginsBlockDuration,
				'Too many failed logins',
				$configFailedLoginsBlockCount . ' failed logins within 1 hour'
			);
		}
		
		if($configFailedLoginsSlowdownCount && $failedLogins >= $configFailedLoginsSlowdownCount) {
			sleep($configFailedLoginsSlowdownDelay);
		}

		if($configFailedLoginsRandomSleep) {
			// Sleep for between 1 second - 2 seconds.
			// This is done to throw off timing attacks.
			$sleep = rand(1, 100) * 10000;
			$sleep += 1000000;
			usleep($sleep);
		}
	}
}