/**
 * Helper function to help with the file upload logic.  There is a small amount of javascript for "prettifying"
 * the upload inputs.  This helps with the selection between
 */
"use strict";

(() => {
	document.addEventListener('click', evt => {
		let match;

		if((match = evt.target.closest('.formfileinput .file-input-upload-button'))) {
			evt.target.closest('.formfileinput').querySelector('input[type=file]').click();
			evt.preventDefault();
		}
		else if((match = evt.target.closest('.formfileinput .file-input-link-button'))) {
			let link = window.prompt('Please enter the URL to upload'),
				group = evt.target.closest('.formfileinput'),
				action = group.querySelector('.file-input-action'),
				out = group.querySelector('.file-input-actions');

			if(link !== null && link !== '') {
				action.value = '_link_://' + link;
				out.innerHTML = 'URL: ' + link.replace(/</g, '').replace(/>/g, '');
			}
			evt.preventDefault();
		}
		else if((match = evt.target.closest('.formfileinput .file-input-remove-button'))) {
			let group = evt.target.closest('.formfileinput'),
				action = group.querySelector('.file-input-action'),
				input = group.querySelector('input[type=file]'),
				out = group.querySelector('.file-input-actions');

			input.value = '';
			action.value = '';
			out.innerHTML = 'None Selected';
			evt.preventDefault();
		}
		else if((match = evt.target.closest('.formfileinput .file-input-browse-button'))) {
			let group = evt.target.closest('.formfileinput'),
				action = group.querySelector('.file-input-action'),
				out = group.querySelector('.file-input-actions'),
				dialog = new Core.ModalDialog();

			dialog.ajaxLoad(
				Core.ROOT_URL + 'mediamanagernavigator?&ajax=1',
				{
					pageLoaded: () => {
						// Override the select file function.
						window.FileNavigator.openFile = file => {
							action.value = '_browse_://' + file.dataset.url;
							out.innerHTML = '<img src="' + file.dataset.preview + '" title="Preview of ' + file.dataset.path + '"/>' + '&nbsp;' + file.dataset.path;
							dialog.close();
						}
					}
				}
			);

			evt.preventDefault();
		}
	});

	document.addEventListener('change', evt => {
		let match;

		if((match = evt.target.closest('.formfileinput input[type=file]'))) {
			let group = evt.target.closest('.formfileinput'),
				action = group.querySelector('.file-input-action'),
				input = group.querySelector('input[type=file]'),
				out = group.querySelector('.file-input-actions');

			if(match.files && match.files.length > 1) {
				out.innerHTML = 'Selected ' + match.files.length + ' files';
				action.value = '_upload_';
			}
			else if(match.files && match.files[0]) {
				out.innerHTML = '<img src="' + window.URL.createObjectURL(match.files[0]) + '" style="max-width:100px; max-height:100px;"/>' +
					'&nbsp;' + match.files[0].name;
				action.value = '_upload_';
			}
			else {
				out.innerHTML = 'None Selected';
				action.value = '';
			}
		}
		else if((match = evt.target.closest('.formfileinput .file-input-action'))) {
			let group = evt.target.closest('.formfileinput'),
				out = group.querySelector('.file-input-actions');

			out.innerHTML = match.value;
		}
	});
})();
