/**
 * Any core javascript that needs to be in the foot of the document.
 * This is just for core systems and functions; components and plugins can have your own file!
 *
 * Note, Core is already defined in the header.
 */

//(function(){

var _getParameterObject = null, _getParameters, types, i;

_getParameters = function(){
	if(_getParameterObject !== null) return _getParameterObject;

	var arr, i, ar2, p1, p2, idx;

	_getParameterObject = {};

	if(!window.location.search) return _getParameterObject;

	arr = window.location.search.slice(1).split('&');
	for(i=0; i<arr.length; i++){
		ar2 = arr[i].split('=');
		ar2[0] = decodeURIComponent(ar2[0]);
		ar2[1] = decodeURIComponent(ar2[1]);

		if(ar2[0].match(/\[.+\]/)){
			idx = ar2[0].indexOf('[');
			p1 = ar2[0].substr(0, idx);
			p2 = ar2[0].substr(idx+1, ar2[0].length-idx-2);
			if(typeof _getParameterObject[p1] == 'undefined') _getParameterObject[p1] = {};
			_getParameterObject[p1][p2] = ar2[1];
		}
		else if(ar2[0].match(/\[\]/)){
			p1 = ar2[0].substr(0, ar2[0].length-2);
			if(typeof _getParameterObject[p1] == 'undefined') _getParameterObject[p1] = [];
			_getParameterObject[p1].push(ar2[1]);
		}
		else{
			_getParameterObject[ar2[0]] = ar2[1];
		}
	}

	return _getParameterObject;
};


/**
 * Create a POST request in the browser without ajax.
 * Allows for a traditional page load, only via POST.
 *
 * This is an older API call with minimal functionality.
 * It is recommended to use Core.Ajax instead.
 *
 * @param {string} u URL to request via POST
 */
Core.PostURL = function(u){
	let doc = window.document,
		form = doc.createElement('form');

	window.document.body.appendChild(form);
	form.action = u;
	form.method = 'POST';
	form.submit();
};


/**
 * Event handler for the "confirm" links.
 *
 * This has been expanded from the inner contents of the onClick to allow
 * for better support with IE.
 *
 * @deprecated
 * @param node
 * @returns {boolean}
 */
Core.ConfirmEvent = function(node){
	var confirmText, href;

	if(typeof node.dataset === 'undefined'){
		// IE < 11 support
		if(node.getAttribute('data-confirm') !== null){
			confirmText = node.getAttribute('data-confirm');
		}
		if(node.getAttribute('data-href') !== null){
			href = node.getAttribute('data-href');
		}
	}
	else{
		// Rest of the world.
		if(node.dataset.confirm !== undefined){
			confirmText = node.dataset.confirm;
		}
		if(node.dataset.href !== undefined){
			href = node.dataset.href;
		}
	}

	//event.stopPropagation();
	if(typeof window.event !== 'undefined'){
		// IE hack
		window.event.cancelBubble = true;
	}

	if(confirmText === '' || confirm(confirmText)){
		Core.PostURL(href);
	}

	return false;
};


Core.Request = {
	/**
	 * Get the GET request parameters (or single parameter)
	 *
	 * @param key
	 *
	 * @return object|string
	 */
	get: function(key) {
		var o = _getParameters();

		if(key){
			return (typeof o[key] != 'undefined') ? o[key] : null;
		}
		else{
			return o;
		}
	},

	/**
	 * Set the GET request parameters based on an object.  (The object is converted to a valid query string).
	 *
	 * @param obj
	 */
	set: function(obj) {
		// gogo jquery!
		if(typeof jQuery != 'undefined'){
			window.location.search = '?' + jQuery.param(obj);
		}
		else{
			// Ummm..... :/
		}
	}
};





$('body').on('click', 'a.ajax-link', function(){
	var $link = $(this),
		title = $link.attr('title') ? $link.attr('title') : 'Page',
		$window = $('<div>Loading ' + title + '...</div>'),
		$body = $('body'),
		width;

	// Calculate the inner width of the window.
	// If too small, then just make the dialog 100% of the width.
	width = $body.width() > 900 ? $body.width() * .8 : $body.width();

	$body.append($window);

	$window.dialog({
		modal: true,
		title: title,
		width: width + 'px',
		// This has been moved up here because it's breaking on the newest version of jQuery when on load.
		position: { my: "center top", at: "center top+50", of: window },
		close: function(){ $window.remove(); }
	});

	$window.load(
		$link.attr('href'),
		function(responseText){
			// This is required to recenter the window.
			//$window.dialog('option', 'position', { my: "center", at: "center", of: window });
			// This is commented out because it seems to be breaking on the newest version of jQuery,
			// at least when on the /mediamanagernavigator system.
		}
	);

	return false;
});


$('body').on('click', 'a.confirm-link', function(event){
	var $this = $(this),
		isAjax = $this.hasClass('ajax'),
		form;

	event.preventDefault();

	if($this.data('confirm') === '' || confirm($this.data('confirm'))) {
		// Either the confirm text was empty or the user approved the confirmation.
		if(isAjax) {
			$.notify('Submitting...', 'info');
			$.post($this.data('href'), '', function(response, status) {
				if(status === 'success' && typeof(response.message) !== 'undefined') {
					$.notify(response.message, response.status);
				}
				else if(status === 'success') {
					$.notify('Successful submission', 'success');
				}
				else {
					$.notify('There was an issue while submitting your request.', 'error');
				}
			});
		}
		else {
			// Traditional POST
			form = window.document.createElement('form');
			window.document.body.appendChild(form);
			form.action = $this.data('href');
			form.method = 'POST';
			form.submit();
		}
	}

	return false;
});

(() => {
	let activeTray = null;

	document.addEventListener('click', evt => {
		let match;

		// Handle the logic for opening and closing trays.
		// This snippet should run on every click regardless, because it needs to close the active tray if one is open.
		if( (match = evt.target.closest('.tray .tray-toggle')) !== null) {

			// User clicked on a control label, mark this control as active!
			let tray = match.closest('.tray');

			if(activeTray !== null) {
				// Active tray was clicked on again, close it
				activeTray.classList.remove('active');
			}

			if(activeTray === tray) {
				// Click event triggered on a tray, but this tray is already active.
				// Instead of running the logic, just mark this as closed.
				activeTray = null;
			}
			else {
				// Determine some position data for this tray
				// it'll be up to CSS to select to use this data, (or not)
				let positions = tray.getBoundingClientRect();
				tray.classList.remove('tray-position-left', 'tray-position-top', 'tray-position-bottom', 'tray-position-right');

				if(positions.x > (document.body.clientWidth / 2)) {
					tray.classList.add('tray-position-right');
				}
				else {
					tray.classList.add('tray-position-left');
				}

				if(positions.y > (document.body.clientHeight / 2)) {
					tray.classList.add('tray-position-bottom');
				}
				else {
					tray.classList.add('tray-position-top');
				}

				tray.classList.add('active');
				activeTray = tray;
			}
		}
		else {
			if(activeTray !== null) {
				// Remove the active tray from the UI.
				activeTray.classList.remove('active');
				activeTray = null;
			}
		}


		// Provide support for loading links in the background via GET, POST, or any other valid type.
		// This serves as a replacement for confirm-links, with new functionality.
		if( (match = evt.target.closest('a[data-target="background"]')) ) {
			let method = match.dataset.method || 'GET',
				url = match.dataset.href || match.href,
				confirmText = match.dataset.confirm || null,
				parameters = {};

			if(confirmText) {
				// Background links support confirmation text before everything runs.
				if(confirm(confirmText) !== true) {
					evt.preventDefault();
					return;
				}
			}

			parameters.method = method;

			if(typeof(match.load) === 'function') {
				parameters.callback = match.load;
			}
			else {
				parameters.callback = (data, contentType) => {
					Core.Reload();
				};
			}

			if(typeof(match.submit) === 'function') {
				// Fire the submit function, if one is registered.
				parameters.preSend = match.submit;
			}

			Core.Ajax(url, parameters).catch(e => {
				console.error(e);
			});

			evt.preventDefault();
		}

		// Provide support for loading links in Modal dialog windows
		// This serves as a replacement for ajax-links, with new functionality.
		if( (match = evt.target.closest('a[data-target="modal"]')) ) {
			let method = match.dataset.method || 'GET',
				url = match.dataset.href || match.href,
				dialog = new Core.ModalDialog(),
				preSend = typeof(match.submit) === 'function' ? match.submit : null,
				callback = typeof(match.load) === 'function' ? match.load : null,
				parameters = {};


			parameters.method = method;
			// Scripts are handled internally.
			parameters.loadScripts = false;

			parameters.preSend = () => {
				if(preSend) {
					preSend();
				}
				dialog.open();
			};

			parameters.callback = (data, contentType) => {
				if(callback) {
					// There's a load available, let them deal with it.
					callback(dialog, data);
				}
				else if(contentType === 'application/json' && typeof(data.body) !== 'undefined') {
					// Body is expected to be HTML data encoded in the JSON stream.
					dialog.frame.innerHTML = data.body;
				}
				else if(contentType === 'text/html') {
					dialog.frame.innerHTML = data;
				}
				else if(contentType === 'text/plain') {
					dialog.frame.innerHTML = '<pre>' + data + '</pre>';
				}
				else {
					throw new Error('What am I supposed to do with the response data?');
				}

				if(typeof(data.scripts) !== 'undefined') {
					// Everything should be loaded now, include the scripts that may be required.
					dialog.setScripts(data.scripts);
				}

				if(typeof(data.scripts_inline) !== 'undefined') {
					// Also include any inline scripts
					dialog.setInlineScripts(data.scripts_inline);
				}
			};

			Core.Ajax(url, parameters).catch(e => {
				dialog.frame.innerHTML = '<p class="message-error">Sorry, but something unexpected happened.  Press ESCAPE to close.</p>';
				console.error(e);
			});

			evt.preventDefault();
		}

		if( (match = evt.target.closest('a[data-method="post"]')) ) {
			Core.PostURL(match.href);
			evt.preventDefault();
		}

		if( (match = evt.target.closest('.collection-item-expand-hint')) ) {
			let area = evt.target.closest('.collapsed');
			area.classList.remove('collapsed');
			area.classList.add('expanded');

			if(typeof(evt.target.dataset.hash) !== 'undefined') {
				window.location.hash = '#' + evt.target.dataset.hash;
			}

			evt.preventDefault();
		}

		if( (match = evt.target.closest('.collection-item-collapse-hint')) ) {
			let area = evt.target.closest('.expanded');
			area.classList.remove('expanded');
			area.classList.add('collapsed');

			if(typeof(evt.target.dataset.hash) !== 'undefined') {
				window.location.hash = '#' + evt.target.dataset.hash;
			}

			evt.preventDefault();
		}
	});
})();


/**
 * Below are some jquery-specific functions, only applicable if jquery is loaded already.
 * This will NOT be required by default because jquery is not a requirement of Core,
 * simply a recommendation.
 */
if(typeof $ !== 'undefined'){
	$(function(){
		var i, types;
		/**
		 * Just a simple script to spruce up the message-* blocks with a bit of flair.
		 */
		types = [
			{ msgclass: 'error',    icon: 'exclamation-circle' },
			{ msgclass: 'info',     icon: 'info-circle' },
			{ msgclass: 'note',     icon: 'asterisk' },
			{ msgclass: 'success',  icon: 'check-circle' },
			{ msgclass: 'tutorial', icon: 'question-circle' },
			{ msgclass: 'warning',  icon: 'exclamation-triangle' }
		];

		for(i in types){
			$('.message-' + types[i].msgclass).each(function(){
				$(this).prepend('<i class="message-background-icon icon icon-' + types[i].icon + '"></i>');
			});
		}

		/**
		 * Fancy collapsible fieldsets
		 */
		$(window).change(function() {
			$('fieldset.collapsible').each(function(){
				var $this = $(this), ls;
				if($this.attr('id')){
					// Lookup if this fieldset is expanded or collapsed in the local storage.
					ls = localStorage.getItem('fieldset-collapsible-' + $this.attr('id'));
					if(ls === '0'){
						$this.addClass('collapsed').children(':not(legend,.fieldset-title)').hide();
					}
					else if(ls === '1'){
						$this.removeClass('collapsed');
						return;
					}
				}

				// If the fieldset doesn't have an ID or just wasn't in cache, go with the default.
				if($this.hasClass('collapsed')){
					$this.children(':not(legend,.fieldset-title)').hide();
				}
			});
		});


		//jQuery('fieldset.collapsible.collapsed').children(':not(legend)').hide();

		$('body').on('click', 'fieldset.collapsible legend,fieldset.collapsible .fieldset-title', function(){
			var el = this,
				$this = $(el),
				$fieldset = $this.closest('fieldset'),
				group = $this.data('group') || null;

			// If the group is present, find any other fieldset title with this group and hide it if it's displayed.
			if(group !== null && group !== '') {
				$('.fieldset-title').each(function(){
					var $subThis = $(this),
						$subFieldset = $subThis.closest('fieldset');

					if($subThis.data('group') === group && this !== el && !$subFieldset.hasClass('collapsed')) {
						$subFieldset.toggleClass('collapsed').children(':not(legend,.fieldset-title)').toggle();
					}
				});
			}

			if(!$fieldset.hasClass('debug-section')) {
				$fieldset.toggleClass('collapsed').children(':not(legend,.fieldset-title)').toggle('fast');
			} else {
				$fieldset.toggleClass('collapsed').children(':not(legend,.fieldset-title)').toggle();
			}

			// Record this setting.
			if($fieldset.attr('id')){
				localStorage.setItem('fieldset-collapsible-' + $fieldset.attr('id'), ($fieldset.hasClass('collapsed') ? '0' : '1'));
			}
		});

		// Call the window change event now, all scripts should be loaded by the time this runs. (hopefully)
		$(window).change();
	});
}
