<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core;

use Core\Filestore\File;
use Core\Filestore\Backends\FileLocal;
use Core\Filestore\Factory as FileFactory;
use DOMNode;
use Exception;

/**
 * Description of ComponentAsset
 *
 * @author charlie
 */
class ComponentAsset {
	public $filename;
	public $source;
	public $sourceFile;
	
	const SOURCE_CUSTOM = 'custom';
	const SOURCE_THEME  = 'theme';
	const SOURCE_LOCAL  = 'local';
	
	public function __construct(string $basedir, DOMNode $node) {
		$theme          = ConfigHandler::Get('/theme/selected');
		$this->filename = $node->getAttribute('filename');

		if(file_exists(ROOT_PDIR . 'themes/custom/' . $this->filename)){
			// There is a file with the same relative name in themes/custom, so use that as the source.
			$this->sourceFile = new FileLocal(ROOT_PDIR . 'themes/custom/' . $this->filename);
			$this->source = self::SOURCE_CUSTOM;
		}
		elseif($theme && file_exists(ROOT_PDIR . 'themes/' . $theme . '/' . $this->filename)){
			// There is a file with the same relative name in themes/custom, so use that as the source.
			$this->sourceFile = new FileLocal(ROOT_PDIR . 'themes/' . $theme . '/' . $this->filename);
			$this->source = self::SOURCE_THEME;
		}
		else{
			// Default is just use the actual theme file.
			$this->sourceFile = new FileLocal($basedir . $this->filename);
			$this->source = self::SOURCE_LOCAL;
		}
	}
	
	public function exists() : bool {
		return $this->getDestination()->exists();
	}
	
	/**
	 * 
	 * @return bool
	 * 
	 * @throws Exception
	 */
	public function install() : bool {
		
		$dest = $this->getDestination();
		
		// If both source and destination are identical, nothing needs to be done.
		if($dest->exists() && $dest->identicalTo($this->sourceFile)){
			return false;
		}
		
		if(!$this->sourceFile->exists()){
			throw new Exception('Component asset ' . $this->sourceFile->getFilename() . ' does not exist, unable to install file!');
		}
		
		if(!$this->sourceFile->isReadable()){
			throw new Exception('Component asset ' . $this->sourceFile->getFilename() . ' could not be read, unable to install file!');
		}
		
		$this->sourceFile->copyTo($dest, true);
		return true;
	}
	
	/**
	 * 
	 * @return bool
	 */
	public function uninstall() : bool {
		
		$dest = $this->getDestination();
		
		if(!$dest->exists()){
			return false;
		}
		
		return $dest->delete();
	}
	
	
	public function getDestination() : File {
		// By using Factory as opposed to FileLocal, the destination path will get auto-resolved.
		// This is because the admin may set the path to something different than default.
		return FileFactory::File($this->filename);
	}
}
