<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core;
use DOMElement;
use Core\ConfigHandler;

/**
 * Description of ComponentPageDefinition
 *
 * @author charlie
 */
class ComponentConfig {
	
	public $componentName;
	public $key;
	public $options;
	public $type;
	public $default;
	public $mapto;
	public $encrypted;
	public $formAttributes;
	
	public function __construct(DOMElement $node, string $component){
		$this->componentName  = $component;
		$this->key            = $node->getAttribute('key');
		$this->type           = $node->getAttribute('type') ?: 'string';
		$this->default        = $node->getAttribute('default');
		$this->mapto          = $node->getAttribute('mapto');
		$this->encrypted      = $node->getAttribute('encrypted') ?: '0';
		
		$this->options        = [];
		$this->formAttributes = [];
		
		foreach($node->childNodes as $child){
			switch($child->nodeName){
				case 'configoption':
					$this->options[] = trim($child->nodeValue);
					break;
				case 'configformattribute':
					$this->formAttributes[ $child->getAttribute('key') ] = trim($child->nodeValue);
					break;
			}
		}
	}
	
	/**
	 * Install this config option into the system and return true if changes were detected.
	 * 
	 * @return bool
	 */
	public function install() : bool {
		
		$m = ConfigHandler::GetConfig($this->key);
		$m->setFromArray([
			'options'         => $this->options,
			'type'            => $this->type,
			'default_value'   => $this->default,
			'mapto'           => $this->mapto,
			'encrypted'       => $this->encrypted,
			'form_attributes' => $this->formAttributes,
			'component'       => $this->componentName,
		]);

		$exists = $m->checkExists();

		// Default from the xml, only if it's not already set.
		if ($m->get('value') === null || !$exists){
			$m->set('value', $this->default);
		}
		
		return ($m->save());
	}
	
	/**
	 * Uninstall this config from the system
	 * 
	 * @return bool
	 */
	public function uninstall() : bool {
		$m = ConfigHandler::GetConfig($this->key);
		return ($m->delete());
	}
}
