<?php


namespace Core;


use Core\Date\DateTime;
use Core\Filestore\Factory;
use Core\Filestore\File;

class CoreStyles {
	private static $_Styles = null;
	private static $_StylesCompiled = null;

	/**
	 * Map of HTML colour names to their hex codes; used because the HTML5 input doesn't appreciate color names.
	 * @var string[] 
	 */
	private static $_ColorNames = [
		'aliceblue' => '#f0f8ff',
		'antiquewhite' => '#faebd7',
		'aqua' => '#00ffff',
		'aquamarine' => '#7fffd4',
		'azure' => '#f0ffff',
		'beige' => '#f5f5dc',
		'bisque' => '#ffe4c4',
		'black' => '#000000',
		'blanchedalmond' => '#ffebcd',
		'blue' => '#0000ff',
		'blueviolet' => '#8a2be2',
		'brown' => '#a52a2a',
		'burlywood' => '#deb887',
		'cadetblue' => '#5f9ea0',
		'chartreuse' => '#7fff00',
		'chocolate' => '#d2691e',
		'coral' => '#ff7f50',
		'cornflowerblue' => '#6495ed',
		'cornsilk' => '#fff8dc',
		'crimson' => '#dc143c',
		'cyan' => '#00ffff',
		'darkblue' => '#00008b',
		'darkcyan' => '#008b8b',
		'darkgoldenrod' => '#b8860b',
		'darkgray' => '#a9a9a9',
		'darkgrey' => '#a9a9a9',
		'darkgreen' => '#006400',
		'darkkhaki' => '#bdb76b',
		'darkmagenta' => '#8b008b',
		'darkolivegreen' => '#556b2f',
		'darkorange' => '#ff8c00',
		'darkorchid' => '#9932cc',
		'darkred' => '#8b0000',
		'darksalmon' => '#e9967a',
		'darkseagreen' => '#8fbc8f',
		'darkslateblue' => '#483d8b',
		'darkslategray' => '#2f4f4f',
		'darkslategrey' => '#2f4f4f',
		'darkturquoise' => '#00ced1',
		'darkviolet' => '#9400d3',
		'deeppink' => '#ff1493',
		'deepskyblue' => '#00bfff',
		'dimgray' => '#696969',
		'dimgrey' => '#696969',
		'dodgerblue' => '#1e90ff',
		'firebrick' => '#b22222',
		'floralwhite' => '#fffaf0',
		'forestgreen' => '#228b22',
		'fuchsia' => '#ff00ff',
		'gainsboro' => '#dcdcdc',
		'ghostwhite' => '#f8f8ff',
		'gold' => '#ffd700',
		'goldenrod' => '#daa520',
		'gray' => '#808080',
		'grey' => '#808080',
		'green' => '#008000',
		'greenyellow' => '#adff2f',
		'honeydew' => '#f0fff0',
		'hotpink' => '#ff69b4',
		'indianred' => '#cd5c5c',
		'indigo' => '#4b0082',
		'ivory' => '#fffff0',
		'khaki' => '#f0e68c',
		'lavender' => '#e6e6fa',
		'lavenderblush' => '#fff0f5',
		'lawngreen' => '#7cfc00',
		'lemonchiffon' => '#fffacd',
		'lightblue' => '#add8e6',
		'lightcoral' => '#f08080',
		'lightcyan' => '#e0ffff',
		'lightgoldenrodyellow' => '#fafad2',
		'lightgray' => '#d3d3d3',
		'lightgrey' => '#d3d3d3',
		'lightgreen' => '#90ee90',
		'lightpink' => '#ffb6c1',
		'lightsalmon' => '#ffa07a',
		'lightseagreen' => '#20b2aa',
		'lightskyblue' => '#87cefa',
		'lightslategray' => '#778899',
		'lightslategrey' => '#778899',
		'lightsteelblue' => '#b0c4de',
		'lightyellow' => '#ffffe0',
		'lime' => '#00ff00',
		'limegreen' => '#32cd32',
		'linen' => '#faf0e6',
		'magenta' => '#ff00ff',
		'maroon' => '#800000',
		'mediumaquamarine' => '#66cdaa',
		'mediumblue' => '#0000cd',
		'mediumorchid' => '#ba55d3',
		'mediumpurple' => '#9370db',
		'mediumseagreen' => '#3cb371',
		'mediumslateblue' => '#7b68ee',
		'mediumspringgreen' => '#00fa9a',
		'mediumturquoise' => '#48d1cc',
		'mediumvioletred' => '#c71585',
		'midnightblue' => '#191970',
		'mintcream' => '#f5fffa',
		'mistyrose' => '#ffe4e1',
		'moccasin' => '#ffe4b5',
		'navajowhite' => '#ffdead',
		'navy' => '#000080',
		'oldlace' => '#fdf5e6',
		'olive' => '#808000',
		'olivedrab' => '#6b8e23',
		'orange' => '#ffa500',
		'orangered' => '#ff4500',
		'orchid' => '#da70d6',
		'palegoldenrod' => '#eee8aa',
		'palegreen' => '#98fb98',
		'paleturquoise' => '#afeeee',
		'palevioletred' => '#db7093',
		'papayawhip' => '#ffefd5',
		'peachpuff' => '#ffdab9',
		'peru' => '#cd853f',
		'pink' => '#ffc0cb',
		'plum' => '#dda0dd',
		'powderblue' => '#b0e0e6',
		'purple' => '#800080',
		'rebeccapurple' => '#663399',
		'red' => '#ff0000',
		'rosybrown' => '#bc8f8f',
		'royalblue' => '#4169e1',
		'saddlebrown' => '#8b4513',
		'salmon' => '#fa8072',
		'sandybrown' => '#f4a460',
		'seagreen' => '#2e8b57',
		'seashell' => '#fff5ee',
		'sienna' => '#a0522d',
		'silver' => '#c0c0c0',
		'skyblue' => '#87ceeb',
		'slateblue' => '#6a5acd',
		'slategray' => '#708090',
		'slategrey' => '#708090',
		'snow' => '#fffafa',
		'springgreen' => '#00ff7f',
		'steelblue' => '#4682b4',
		'tan' => '#d2b48c',
		'teal' => '#008080',
		'thistle' => '#d8bfd8',
		'tomato' => '#ff6347',
		'turquoise' => '#40e0d0',
		'violet' => '#ee82ee',
		'wheat' => '#f5deb3',
		'white' => '#ffffff',
		'whitesmoke' => '#f5f5f5',
		'yellow' => '#ffff00',
		'yellowgreen' => '#9acd32',
	];

	/**
	 * Get all custom css settings that are available, along with their values
	 *
	 * @return array
	 *
	 * @throws \Exception
	 */
	public static function GetStyles() : array {
		self::_LoadStyles();
		return self::$_Styles;
	}

	public static function GetThemeStyles() : array {
		$settingsFileTheme = Factory::File(Theme::GetTheme()->getBaseDir() . 'assets/css/settings.css');
		if($settingsFileTheme->exists()) {
			return self::_ProcessSettingsFile($settingsFileTheme);
		}
		else {
			return [];
		}
	}

	public static function GetCustomStyles() : array {
		$settingsFileCustom = Factory::File(ROOT_PDIR . 'themes/custom/assets/css/settings.css');
		if($settingsFileCustom->exists()) {
			return self::_ProcessSettingsFile($settingsFileCustom);
		}
		else {
			return [];
		}
	}

	/**
	 * Get the computed and rendered CSS string for the settings
	 *
	 * @return string
	 * @throws \Exception
	 */
	public static function GetStylesAsCSS() : string {
		if(self::$_StylesCompiled !== null) {
			// Use memory cache for repeated calls
			return self::$_StylesCompiled;
		}

		$cacheKey = 'core-css-settings-compiled';
		if(DEVELOPMENT_MODE) {
			// Development mode has cache here disabled.
			$cache = null;
		}
		else {
			$cache = Cache::Get($cacheKey);
		}

		if($cache) {
			// Use Core cache for persistence between calls
			self::$_StylesCompiled = $cache;
			return self::$_StylesCompiled;
		}
		else {
			$settings = self::GetStyles();

			self::$_StylesCompiled = ':root{';
			foreach($settings as $key => $val) {
				self::$_StylesCompiled .= '--' . $key . ':' . $val . ';';
			}
			self::$_StylesCompiled .= "}";

			Cache::Set($cacheKey, self::$_StylesCompiled, SECONDS_ONE_MONTH);
			return self::$_StylesCompiled;
		}
	}

	/**
	 * Get an individual key
	 *
	 * If it's undefined, an empty string is returned.
	 *
	 * This will automatically map HTML color names and CSS shortcodes to the full version
	 * for example, "White" gets converted to "#ffffff" and "#DDD" gets converted to "#dddddd"
	 *
	 * @param string $key
	 * @return string
	 */
	public static function GetStyle(string $key) : string {
		self::_LoadStyles();
		$value = self::$_Styles[ $key ] ?: '';

		if(isset(self::$_ColorNames[ strtolower($value) ])) {
			$value = self::$_ColorNames[ strtolower($value) ];
		}
		elseif(preg_match('/^#[A-Fa-f0-9]{3}$/', $value)) {
			$value = strtolower(preg_replace('/^#([A-Fa-f0-9])([A-Fa-f0-9])([A-Fa-f0-9])$/', '#\1\1\2\2\3\3', $value));
		}

		return $value;
	}

	/**
	 * Add the styles for user-configurable styles
	 */
	public static function AttachCoreStyleConfigs(View $view) {
		$view->addStyle('<style data-source="settings">' . self::GetStylesAsCSS() . '</style>');
	}

	/**
	 * Save a set of styles into the custom stylesheet
	 *
	 * @param array $styles
	 */
	public static function SaveStyles(array $styles) {
		$customStyles = array_merge(self::GetCustomStyles(), $styles);

		$fileHeader = '/**
 * User-definable settings, edit this as necessary!
 *
 * Last updated ' . DateTime::Now('FDT') . '
 */';

		$fileContents = $fileHeader . "\n\n" . ':root {';

		foreach($customStyles as $key => $val) {
			$fileContents .= "\n\t--" . $key . ': ' . $val . ';';
		}

		$fileContents .= "\n" . '}' . "\n";

		$destFile = Factory::File(ROOT_PDIR . 'themes/custom/assets/css/settings.css');
		$saved = $destFile->putContents($fileContents);

		if(!$saved) {
			throw new \Exception('Unable to save styles, check permissions on themes/custom!');
		}

		$cacheKey = 'core-css-settings';
		Cache::Delete($cacheKey);
	}

	/**
	 * Get all custom css settings that are available, along with their values
	 *
	 * @throws \Exception
	 */
	private static function _LoadStyles() {
		if(self::$_Styles !== null) {
			// Use memory cache for repeated calls
			return;
		}

		$cacheKey = 'core-css-settings';
		if(DEVELOPMENT_MODE) {
			// Development mode has cache here disabled.
			$cache = null;
		}
		else {
			$cache = Cache::Get($cacheKey);
		}

		if($cache) {
			// Use Core cache for persistence between calls
			self::$_Styles = $cache;
		}
		else {
			// We check both files because it's very possible that with updates to the firmware,
			// new themes installed, or other reasons.. the custom settings.css file may not contain
			// all expected keys within the theme settings file.
			$settingsFileCustom = Factory::File(ROOT_PDIR . 'themes/custom/assets/css/settings.css');
			try {
				$settingsFileTheme = Factory::File(Theme::GetTheme()->getBaseDir() . 'assets/css/settings.css');
			}
			catch(\Exception $ex) {
				$settingsFileTheme = null;
			}


			if($settingsFileTheme && $settingsFileTheme->exists() && $settingsFileCustom->exists()) {
				self::$_Styles = array_merge(
					self::_ProcessSettingsFile($settingsFileTheme),
					self::_ProcessSettingsFile($settingsFileCustom)
				);
			}
			elseif($settingsFileTheme && $settingsFileTheme->exists()) {
				self::$_Styles = self::_ProcessSettingsFile($settingsFileTheme);
			}
			elseif($settingsFileCustom->exists()) {
				self::$_Styles = self::_ProcessSettingsFile($settingsFileCustom);
			}
			else {
				self::$_Styles = [];
			}

			Cache::Set($cacheKey, self::$_Styles, SECONDS_ONE_MONTH);
		}
	}

	/**
	 * Process a file for its CSS settings
	 *
	 * @param File $file
	 * @return string[]
	 */
	private static function _ProcessSettingsFile(File $file) : array {
		$keys = [];
		$ret = [];
		$contents = $file->getContents();

		preg_match_all('/--([^:]*):([^;]*);/', $contents, $keys);

		$matchLen = count($keys[1]);
		for($i = 0; $i < $matchLen; $i++) {
			$ret[ $keys[1][$i] ] = trim($keys[2][$i]);
		}

		return $ret;
	}
}