<?php


namespace Core;


use Core\Templates\Template;

class PaneViewer {
	/** @var Model */
	protected $_model;

	/** @var ViewControls */
	protected $_links;

	protected $_name;

	public function setModel(Model $model) {
		$this->_model = $model;
	}

	// <editor-fold desc="Control & Link Methods">

	/**
	 * Add a control to this interface to display inline
	 *
	 * @param ViewControl $control
	 */
	public function addControl(ViewControl $control) {
		if($this->_links === null) {
			$this->_links = new ViewControls();
		}


		$this->_links->addLink($control);
	}

	/**
	 * Add a control to this table via its title, link, and icon parameters.
	 *
	 * This provides support for Core-2018 and prior.
	 *
	 * @param string $title
	 * @param string $link
	 * @param string|null $icon
	 */
	public function addControlTLI(string $title, string $link, string $icon = null) {
		$this->addControl(ViewControl::FactoryTLI($title, $link, $icon));
	}

	public function getControls() : ViewControls {
		if($this->_links === null) {
			$this->_links = new ViewControls();
		}

		return $this->_links;
	}

	// </editor-fold>

	public function getName() : string {
		return $this->_name ?: str_replace('\\', '_', strtolower(get_class($this)));
	}

	/**
	 * Render this paneviewer as HTML
	 *
	 * @return string
	 */
	public function render() : string {
		$tpl = Template::Factory('includes/paneviewer/main');

		$tpl->assign('pane', $this);
		$tpl->assign('controls', $this->getControls());

		return $tpl->fetch();
	}

	protected function _setupLinks() {

	}
}