<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\CLI;

use Core\Router;

/**
 * Description of LogSink
 *
 * @author charlie
 */
class LogSink implements \Core\Logger\LogSinkInterface {
	
	public function getDefaultLevel() : int{
		// By default the STDOUT/CLI logger is disabled.
		return 0;
	}
	
	public function getMaximumLevel() : int{
		// STDOUT/CLI accepts any level.
		return LOG_LEVEL_TRACE;
	}
	
	public function log(\Core\Logger\LogEntry $entry){
		// Format the line prefix for the profiler

		if(EXEC_MODE === 'WEB' && !headers_sent()) {
			// If no headers have been sent yet and the execution mode is set to the web,
			// just return without printing anything.  This prevents the "content already sent" error should headers need to be sent.
			return;
		}

		if(EXEC_MODE === 'WEB' && !(defined('DEVELOPMENT_MODE') && DEVELOPMENT_MODE)) {
			// DEV mode is either not set or not enabled, so do not print anything to the screen.
			return;
		}

        $view = Router::$DefaultView;

		if(EXEC_MODE === 'WEB' && $view && $view->contenttype !== 'text/html') {
			// Skip printing when rendering in the web on a page other than an HTML page.
			return;
		}
		
		if(EXEC_MODE === 'WEB') {
			// Wrap this entry in some targetable styles!
			echo '<pre class="log-entry log-level-' . $entry->level . ' log-type-' . $entry->type . '">';
			if($entry->icon){
				echo '<i class="icon icon-' . $entry->icon . '"></i>  ';
			}
		}
		
		switch($entry->level){
			case LOG_LEVEL_CRITICAL:
				CLI::PrintError($entry->message);
				break;
			case LOG_LEVEL_HIGH:
				CLI::PrintWarning($entry->message);
				break;
			case LOG_LEVEL_DEBUG:
			case LOG_LEVEL_TRACE:
				CLI::PrintLine($entry->message, COLOR_DEBUG);
				break;
			default:
				CLI::PrintLine($entry->message);
		}
		
		if($entry->details){
			CLI::PrintLine($entry->details);
		}
		
		if($entry->trace){
			CLI::PrintDebug(array_map(function($a){ return $a['formatted']; }, $entry->trace));
		}
		
		if(EXEC_MODE === 'WEB'){
			echo '</pre>';
		}
	}
}
