<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Datamodel;

/**
 * Description of DatasetSelect
 *
 * @author charlie
 */
class DatasetSelect {
	/**
	 * Table name or alias to query the data
	 *
	 * This is optional if there is only one table or the column is not ambigious.
	 *
	 * @var string
	 */
	public $table;

	/**
	 * Column to select, either this, function, or formula is required.
	 *
	 * @var string
	 */
	public $column;

	/**
	 * Alias to return the selected data
	 *
	 * Supported across column, function, and formula
	 *
	 * @var string
	 */
	public $alias;

	/**
	 * A function to call, must be supported by the underlying DMI layer.
	 *
	 * Some functions allow for a column to be selected,
	 * where the column attribute can be combined with this option.
	 *
	 * @var string
	 */
	public $function;

	/**
	 * A preformatted formula to run as the SELECT query
	 *
	 * This is EXTREMELY DANGEROUS to use and may break in weird ways!
	 *
	 * @var string
	 */
	public $formula;

	/**
	 * List of parameters to use for the formula
	 *
	 * @var array
	 */
	public $formulaParams = [];

	/**
	 * A subquery to perform within another query,
	 * the output of this will be used to populate the alias variable in the database server.
	 *
	 * @var Dataset
	 */
	public $subquery;

	
	public function __construct(string $select = null) {
		if($select) {
			$this->column($select);
		}
	}

	public function alias(string $alias) : self {
		$this->alias = $alias;
		return $this;
	}

	public function column(string $column) : self {
		$select = trim($column);

		if(($pos = strpos($select, '.')) !== false){
			$this->table = substr($select, 0, $pos);
			$select = substr($select, $pos + 1);
		}

		if(($pos = strpos($select, ' ')) !== false){
			$this->alias = substr($select, $pos + 1);
			$select = substr($select, 0, $pos);
		}

		$this->column = $select;

		return $this;
	}

	public function subQuery(Dataset $query) : self {
		$this->subquery = $query;
		return $this;
	}

	public function func(string $function) : self {
		$this->function = $function;
		return $this;
	}

	public static function Init() : self {
		return new DatasetSelect();
	}
}
