<?php
/**
 * File for class Schema definition in the coreplus project
 * 
 * @package Core\Datamodel
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20131022.1655
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\Datamodel\Columns;
use Core\Model;


class SchemaColumn_ISO_8601_datetime extends SchemaColumn {
	public function __construct(){
		// Defaults
		$this->typecast = 'string';
		$this->type     = Model::ATT_TYPE_ISO_8601_DATETIME;
		$this->default  = '0000-00-00 00:00:00';
		$this->comment  = 'Datetime in YYYY-mm-dd HH:ii:ss format';
		$this->formAttributes['datepicker_dateformat'] = 'yy-mm-dd';
		$this->formAttributes['datetimepicker_timeformat'] = 'HH:mm';
		$this->formAttributes['saveformat'] = 'Y-m-d H:i:00';
		$this->formAttributes['type'] = 'datetime';
		$this->formAttributes['access'] = '*';
	}

	/**
	 * Set the value from the database for this column
	 *
	 * Handles all translations and conversions as necessary.
	 *
	 * @param mixed $val
	 */
	public function setValueFromDB($val) {
		$this->valueDB = $val;
		$this->value = $val;

		if($this->encrypted) {
			// Decrypt the value first.
			$val = Model::DecryptValue($val);
		}

		if($val === '' || $val === '0000-00-00 00:00:00' || $val === null) {
			// Ensure correct return values
			$val = $this->null ? null : '0000-00-00 00:00:00';
		}

		$this->valueTranslated = $val;
	}

	/**
	 * Set the value from the application/userspace for this column
	 *
	 * Handles all translations and conversions as necessary.
	 *
	 * @param mixed $val
	 */
	public function setValueFromApp($val) {
		$this->valueTranslated = $val;

		if($val === '' || $val === '0000-00-00 00:00:00' || $val === null) {
			$val = $this->null ? null : $this->default;
		}

		if($this->encrypted) {
			// Decrypt the value last.
			$val = Model::EncryptValue($val);
		}

		$this->value = $val;
	}
}