<?php
/**
 * File for class FormTabsGroup definition in the coreplus project
 *
 * @package Core\Forms
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\Forms;



use Exception;

/**
 * A short teaser of what FormTabsGroup does.
 *
 * More lengthy description of what FormTabsGroup does and why it's fantastic.
 *
 * <h3>Usage Examples</h3>
 *
 *
 * @todo Write documentation for FormTabsGroup
 * <h4>Example 1</h4>
 * <p>Description 1</p>
 * <code>
 * // Some code for example 1
 * $a = $b;
 * </code>
 *
 *
 * <h4>Example 2</h4>
 * <p>Description 2</p>
 * <code>
 * // Some code for example 2
 * $b = $a;
 * </code>
 *
 * 
 * @author Charlie Powell <charlie@evalagency.com>
 *
 */
class ButtonsGroup extends FormGroup {

	public function __construct($atts = null) {
		parent::__construct($atts);

		$this->set('class', 'formgroup formgroup-buttons');
	}

	public function getTemplateName() {
		return 'forms/groups/buttons.tpl';
	}

	/**
	 * Add a given element, (or element type with attributes), onto this form or form group.
	 *
	 * @param            $element
	 * @param null|array $atts
	 */
	public function addElement($element, $atts = null) {
		// Since this allows for just plain names to be submitted, translate
		// them to the form object to be rendered.

		if(is_scalar($element)) {
			// The element is the key, so map it.
			$element = FormElement::Factory($element);
		}

		if(!($element instanceof ButtonInput || $element instanceof SubmitInput || $element instanceof ResetInput)) {
			throw new Exception('Only buttons can be added to a button group! (A ' . get_class($element) . ' was attempted to be added');
		}

		if($atts) {
			$element->setFromArray($atts);
		}

		$this->_elements[] = $element;
	}
}