<?php
/**
 * Class file for FormTextareaInput
 *
 * @package Core\Forms
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\Forms;


/**
 * Class MessageBox provides a basic message area that can be rendered in a form
 *
 * Just a convenience element for the controller to add messages in groups or along with elements.
 *
 * The keys supported here for attributes:
 *
 * * message_type - error, info, warning, or any other type; affects the class
 * * message - Text of the message to display
 * * id - standard HTML ID
 * * class - Optional additional class names
 * * value - alias of message
 *
 * @package Core\Forms
 */
class MessageBox extends FormElement {
	public function  __construct($atts = null) {
		parent::__construct($atts);

		// Some defaults
		$this->_attributes['class'] = 'formelement formmessagebox';
		$this->_validattributes     = array('style');
	}

	public function getValue() {
		if(isset($this->_attributes['value'])) {
			$value = $this->_attributes['value'];
		}
		elseif(isset($this->_attributes['message'])) {
			$value = $this->_attributes['message'];
		}
		else {
			$value = 'No Message Set';
		}

		// Support I18N
		if(strpos($value, 't:') === 0) {
			$value = t(substr($value, 2));
		}

		return $value;
	}
}
