<?php
/**
 * File for class Key definition in the coreplus project
 * 
 * @package Core\GPG
 * @author Charlie Powell <charlie@evalagency.com>
 * @date 20140319.1923
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

namespace Core\GPG;


/**
 * A short teaser of what Key does.
 *
 * More lengthy description of what Key does and why it's fantastic.
 *
 * <h3>Usage Examples</h3>
 *
 *
 * @todo Write documentation for Key
 * <h4>Example 1</h4>
 * <p>Description 1</p>
 * <code>
 * // Some code for example 1
 * $a = $b;
 * </code>
 *
 *
 * <h4>Example 2</h4>
 * <p>Description 2</p>
 * <code>
 * // Some code for example 2
 * $b = $a;
 * </code>
 *
 * 
 * @package Core\GPG
 * @author Charlie Powell <charlie@evalagency.com>
 *
 */
class SecretKey extends PrimaryKey {
	/**
	 * Get the ASCII representation of this key
	 *
	 * @return string
	 *
	 * @throws \Exception
	 */
	public function getAscii(){
		$gpg = new GPG();
		$result = $gpg->_exec('--export-secret-keys -a ' . $this->fingerprint);

		if($result['return'] != 0){
			throw new \Exception(trim($result['error']));
		}

		return $result['output'];
	}

	/**
	 * Get the binary representation of this key
	 *
	 * @return string The binary data of the key
	 *
	 * @throws \Exception
	 */
	public function getBinary() {
		$gpg = new GPG();
		$result = $gpg->_exec('--export-secret-keys ' . $this->fingerprint);

		if($result['return'] != 0){
			throw new \Exception(trim($result['error']));
		}

		return $result['output'];
	}
} 