<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Installer\Checks;

use Core\Installer\InstallHelper;

/**
 * Description of MysqlServerCheck
 *
 * @author charlie
 */
class MysqlServerCheck implements CheckInterface {
	
	private $_server;
	
	public function __construct(string $server) {
		$this->_server = $server;
	}
	
	public function getDetails(): string {
		return 'This app requires a connection to the MySQL server at ' . $this->_server;
	}

	public function getFix(): string {
		if(
			$this->_server === 'localhost' ||
			strpos($this->_server, '127.') === 0
		){
			return InstallHelper::GetPackageInstallCommand() . ' mariadb-server';
		}
		else{
			return '';
		}
	}

	public function getTitle(): string {
		return 'MySQL Server on ' . $this->_server;
	}

	/**
	 * Get additional details about this check, but not necessarily about the fix.
	 * @return string
	 */
	public function getInformation() : string {
		return '';
	}

	public function test(): bool {
		// Just check for a 2003 code here; that indicates that the mysql server isn't running on that machine.
		// ERROR 2003
		$out = trim(exec('mysql --connect-timeout=1 -h ' . $this->_server . ' -e quit 2>&1'));
		
		if(strpos($out, 'ERROR 2003') !== false){
			// Server not accessable.
			if(EXEC_MODE === 'CLI' && ($fix = $this->getFix()) !== ''){
				// Auto-install!
				$ret = null;
				$out = [];
				exec($fix, $out, $ret);
				return ($ret === 0);
			}
			else{
				return false;
			}
		}
		// @todo any other error codes?
		
		return true;
	}

	public function testExtended(): string {
		return $this->test() ? self::STATUS_PASSED : self::STATUS_FAILED;
	}
}
