<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Installer\Checks;

use Core\Installer\InstallHelper;

/**
 * Description of PHPExtensionCheck
 *
 * @author charlie
 */
class NativePackageCheck {
	
	private $_package;
	private $_cmd;
	
	public function __construct(string $package, string $cmd = null) {
		$this->_package = $package;
		$this->_cmd = $cmd !== null ? $cmd : $package;
	}
	
	/**
	 * Get the title of this check
	 * @return string
	 */
	public function getTitle() : string {
		return 'Native Package ' . $this->_package;
	}

	
	/**
	 * Run the test for this check
	 * @return bool True if passed, False otherwise
	 */
	public function test() : bool {
		
		$available = trim(exec('which ' . $this->_cmd . ' 2>/dev/null')) !== '';
		
		if($available){
			return true;
		}
		elseif(EXEC_MODE === 'CLI' && ($fix = $this->getFix()) !== ''){
			// AUTO-INSTALL!
			$ret = null;
			$out = [];
			exec($fix, $out, $ret);
			return ($ret === 0);
		}
		else{
			return false;
		}
	}

	public function testExtended(): string {
		return $this->test() ? self::STATUS_PASSED : self::STATUS_FAILED;
	}
	
	/**
	 * Get additional details that may help the user fix the problem
	 * @return string
	 */
	public function getDetails() : string {
		if($this->test()) {
			return '';
		}
		else {
			// Automatic support is supported!
			return 'Install ' . $this->_package . ' from your system package manager';
		}
	}
	
	/**
	 * Get the scripted fix for this problem that provides the user how to fix it
	 * @return string
	 */
	public function getFix(): string {
		$fixes = [];
		
		$fixes[] = InstallHelper::GetPackageInstallCommand() . ' ' . $this->_package;
		
		return implode(NL, $fixes);
	}
}
