<?php


namespace Core\ListingTable;


use Core\Request;

class Action {
	/** @var null|string Content type match for this action */
	public $ctype = null;
	/** @var array Key/value pairs of GET tags to match */
	public $matches = [];
	/** @var array Function on the backend to call on a matched action */
	public $call = null;
	/** @var null|string Extension to match for this action */
	public $ext = null;

	/**
	 * Check if this request matches the given page request
	 *
	 * @param Request $request
	 * @return bool
	 */
	public function checkMatches(Request $request, array $parameters = null) : bool {
		if($this->ctype !== null && $this->ctype !== $request->ctype) {
			return false;
		}

		if($this->ext !== null && $this->ext !== $request->ext) {
			return false;
		}

		foreach($this->matches as $k => $v) {
			if(!(is_array($parameters) && isset($parameters[ $k ]) && $parameters[ $k ] === $v)) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Get the number of matching points for this action, useful to know if one action is more specific than another
	 *
	 * @return int
	 */
	public function getMatchWeight() : int {
		return (count($this->matches) << 1) + ($this->ctype !== null ? 1 : 0) + ($this->ext !== null ? 1 : 0);
	}

	public function execute() {
		call_user_func($this->call);
	}
}