<?php

namespace Core\ListingTable\Renderer;

use Core\ListingTable\Column;
use Core\ListingTable\Table;
use Core\ListingTable\TableInterface;
use Core\Model;
use Core\Templates\Template;

/**
 * Class TableRenderer
 *
 * Renders data in a "table" like format, with rows and cells.
 *
 * @package Core\ListingTable\Renderer
 */
class TableRenderer implements RendererInterface {
	private $_allColumns;
	private $_columns;
	private $_bulkActions = false;
	private $_page;
	private $_total;
	private $_limit;
	private $_records;
	private $_hasFilters;
	private $_filtersSet;

	/**
	 * TableRenderer constructor.
	 *
	 * @param TableInterface $table
	 */
	public function __construct(TableInterface $table) {

		// Set the necessary information from the Table
		$this->_allColumns  = $table->getColumns();
		$this->_columns     = $table->getColumnsVisible();
		$this->_bulkActions = $table->checkHasBulkActions();
		$this->_page        = $table->getPage();
		$this->_total       = $table->getTotalCount();
		$this->_limit       = $table->getLimit();
		$this->_records     = $table->getRecords();
		$this->_hasFilters  = $table->checkHasFilters();
		$this->_filtersSet  = $table->checkFiltersSet();
	}

	/**
	 * Render the contents of this interface with all the requested data
	 *
	 * @return array
	 *         +pager   bool   REQUIRED true/false if the pager is enabled
	 *         +options string REQUIRED HTML output of options for this interface
	 *         +body    string REQUIRED HTML output of the body
	 *         +page    int    Current page number, required if pager is set to true
	 *         +total   int    Total number of results, required if pager is set to true
	 *         +pages   int    Total number of pages, required if pager is set to true
	 */
	public function render() : array {

		$pages = (int) ceil($this->_total / $this->_limit);
		$template = Template::Factory('includes/listingtable/table-data');

		$template->assign('pages', $pages);
		$template->assign('page', $this->_page);
		$template->assign('total', $this->_total);
		$template->assign('limit', $this->_limit);
		$template->assign('columns', $this->_columns);
		$template->assign('all_columns', $this->_allColumns);
		$template->assign('records', $this->_records);
		$template->assign('bulk_actions', $this->_bulkActions);

		return [
			'body' => $template->fetch(),
			'download' => 'csv',
		];
	}
}