<?php

namespace Core\Page;

use Core\ListingTable\Table;
use Core\FilterForm;
use Core\Core;
use Core\Datamodel\Dataset;
use PageMetaModel;
use PageModel;

class PageTable extends Table {
	
	public function __construct() {
		$this->setModelName('PageModel');

		parent::__construct();
	}

	protected function _setupFilters() {
		$components = Core::GetComponents();

		$componentopts = ['' => '-- ' . t('STRING_VIEW_ALL_COMPONENTS') . ' --'];
		foreach($components as $c){
			/** @var Component $c */
			$componentopts[$c->getKeyName()] = $c->getName();
		}
		// Sort them by name!
		asort($componentopts);

		// Load all the tags on the site from the various pages and load them as a filter.
		$pageMetas = Dataset::Init()
			->select('meta_value', 'meta_value_title')
			->table('page_meta')
			->where('meta_key = keyword')
			->unique(true)
			->order('meta_value_title ASC')
			->executeAndGet();
		$pageMetaOptions = ['' => '-- All Tags --'];
		foreach($pageMetas as $dat){
			$pageMetaOptions[$dat['meta_value']] = $dat['meta_value_title'];
		}

		$this->addFilter(
			'text',
			[
				'name' => 'title',
				'title' => t('STRING_TITLE'),
				'link' => FilterForm::LINK_TYPE_CONTAINS,
			]
		);

		$this->addFilter(
			'text',
			[
				'name' => 'rewriteurl',
				'title' => t('STRING_URL'),
				'link' => FilterForm::LINK_TYPE_CONTAINS,
			]
		);

		$this->addFilter(
			'text',
			[
				'name' => 'parenturl',
				'title' => t('STRING_PARENT_URL'),
				'link' => FilterForm::LINK_TYPE_STARTSWITH,
			]
		);

		$this->addFilter(
			'select',
			[
				'name' => 'component',
				'title' => t('STRING_COMPONENT'),
				'options' => $componentopts,
				'link' => FilterForm::LINK_TYPE_STANDARD,
			]
		);

		$this->addFilter(
			'select',
			[
				'name' => 'page_types',
				'title' => t('STRING_INCLUDE_ADMIN_PAGES'),
				'options' => ['all' => t('STRING_VIEW_ALL_PAGES'), 'no_admin' => t('STRING_EXCLUDE_ADMIN_PAGES')],
				'value' => 'no_admin',
			]
		);

		if(sizeof($pageMetaOptions) > 1){
			$this->addFilter(
				'select',
				[
					'name' => 'keyword',
					'title' => 'Page Keyword',
					'options' => $pageMetaOptions
				]
			);
		}
	}

	protected function _processFilters() {
		parent::_processFilters();

		if($this->getFilterValue('page_types') == 'no_admin'){
			$this->getModelFactory()->where('admin = 0');
			$this->getModelFactory()->where('selectable = 1');
		}

		if($this->getFilterValue('keyword')){
			$pageMetas = PageMetaModel::FindRaw(
				['meta_value = ' . $this->getFilterValue('keyword'), 'meta_key = keyword']
			);
			$pageURLs = [];
			foreach($pageMetas as $row){
				$pageURLs[] = $row['baseurl'];
			}
			$this->getModelFactory()->where('baseurl IN ' . implode(',', $pageURLs));
		}
	}

	protected function _setupColumns() {
		$this->addColumn('title');
		$this->addColumn('rewriteurl');
		$this->addColumn('pageviews')->hidden();
		$this->addColumn('popularity');
		$this->addColumn('expires')->hidden();
		$this->addColumn('created')->hidden();
		$this->addColumn('updated')->hidden();
		$this->addColumn('published_status');
		$this->addColumn('published');
		$this->addColumn('published_expires');
		$this->addColumn('seotitle');
		$this->addColumn('teaser')->hidden();
		$this->addColumn('access');
		$this->addColumn('component')->hidden();
	}
}
