<?php
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\Utilities;

use Core\ConfigHandler;

/**
 * Description of PasswordGenerator
 *
 * @author charlie
 */
class PasswordGenerator {
	/**
	 * @var int Minimum length of password to generate.
	 */
	public $minLength;
	
	/**
	 * @var int Minimum amount of symbols to include, set to 0 to disable
	 */
	public $symbols;
	
	/**
	 * @var int Minimum amount of capital letters to include, set to 0 to disable
	 */
	public $capitals;
	
	/**
	 * @var int Minimum amount of numbers to include, set to 0 to disable
	 */
	public $numbers;
	
	/**
	 * @var bool Set to true to omit ambiguous letters
	 */
	public $ambiguousLetters = false;
	
	public function __construct() {
		// Defaults based on the site configuration
		$this->minLength = max(ConfigHandler::Get('/user/password/minlength'), 20);
		$this->symbols   = max(ConfigHandler::Get('/user/password/requiresymbols'), 1);
		$this->capitals  = max(ConfigHandler::Get('/user/password/requirecapitals'), 2);
		$this->numbers   = max(ConfigHandler::Get('/user/password/requirenumbers'), 2);
	}
	
	/**
	 * Generate a relatively strong password using the assigned parameters
	 * 
	 * @return string
	 */
	public function generate() : string {
		return $this->_gen();
	}
	
	/**
	 * Generate a hex string, optionally case sensitive.
	 * 
	 * @return string
	 */
	public function generateHex(bool $caseSensitive = false) : string {
		
		$keyspace = $caseSensitive ? '0123456789abcdefABCDEF' : '0123456789ABCDEF';
		
		return $this->_genFromKeyspace($keyspace, $this->minLength);
	}
	
	/**
	 * 1-liner method to quickly generate a password based on site defaults
	 * 
	 * @return string
	 */
	public static function QuickGen() : string {
		$generator = new PasswordGenerator();
		return $generator->generate();
	}
	
	/**
	 * Internal method to perform the actual generation
	 * 
	 * @return string
	 */
	private function _gen() : string {
		$sym = $this->symbols ? rand($this->symbols, $this->symbols + 4) : 0;
		$cap = $this->capitals ? rand($this->capitals, $this->capitals + 6) : 0;
		$num = $this->numbers ? rand($this->numbers, $this->numbers + 6) : 0;
		
		$let = max($this->minLength - $sym - $cap - $num, 10);
		$let = rand($let, $let + 6);
		
		if($this->ambiguousLetters) {
			$symKeyspace = '!@#$%^&*()_+~';
			$capKeyspace = 'QWERTYUPKJHGFDSAZXCVBNM';
			$numKeyspace = '23456789';
			$letKeyspace = 'qwertyupkjhgfdsazxcvbnm';
		}
		else{
			$symKeyspace = '!@#$%^&*()_+=-{}[]|;:"\'<>?,./~`';
			$capKeyspace = 'QWERTYUIOPLKJHGFDSAZXCVBNM';
			$numKeyspace = '0123456789';
			$letKeyspace = 'qwertyuioplkjhgfdsazxcvbnm';
		}
		
		return str_shuffle(
			$this->_genFromKeyspace($symKeyspace, $sym) .
			$this->_genFromKeyspace($capKeyspace, $cap) .
			$this->_genFromKeyspace($numKeyspace, $num) .
			$this->_genFromKeyspace($letKeyspace, $let)
		);
	}
	
	/**
	 * Pull a random character from a given keyspace
	 * 
	 * @param string $keyspace
	 * @param int $length
	 * @return string
	 */
	private function _genFromKeyspace($keyspace, $length) : string {
		$chars = strlen($keyspace) - 1;
		$pw = '';
		$sodium = function_exists('\Sodium\randombytes_uniform');
		
		for($i = 0; $i < $length; $i++ ){
			if($sodium) {
				$pw .= $keyspace[ \Sodium\randombytes_uniform($chars) ];
			}
			else{
				$pw .= $keyspace[ rand(0, $chars) ];
			}
		}
		
		return $pw;
	}
}
