<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\ViewMeta;

/**
 * Defines the behavior that cooperative crawlers, or "robots", should use with the page.
 *
 * @author charlie
 */
class Robots extends GenericMeta {
	protected $_name = 'robots';

	protected $_index = true;

	protected $_follow = true;

	protected $_archive = true;

	protected $_snippet = true;

	/**
	 * Get this Meta's value as a string
	 *
	 * @return string
	 */
	public function __toString() : string {
		$fragments = [];

		if(!$this->_index) {
			$fragments[] = 'noindex';
		}

		if(!$this->_follow) {
			$fragments[] = 'nofollow';
		}

		if(!$this->_archive) {
			$fragments[] = 'noarchive';
		}

		if(!$this->_snippet) {
			$fragments[] = 'nosnippet';
		}

		return implode(', ', $fragments);
	}

	/**
	 * Set the Meta value
	 *
	 * NOTE, the valueKey on the Robots meta parameter HAS NO FUNCTION and is included to preserve functionality
	 *
	 * @param string|bool $value
	 * @param null $valueKey NO USE!
	 */
	public function setContent($value, $valueKey = null) {
		switch($value) {
			case 'noindex':
				$this->_index = false;
				break;
			case 'index':
				$this->_index = true;
				break;
			case 'nofollow':
				$this->_follow = false;
				break;
			case 'follow':
				$this->_follow = true;
				break;
			case 'noarchive':
				$this->_archive = false;
				break;
			case 'archive':
				$this->_archive = true;
				break;
			case 'nosnippet':
				$this->_snippet = false;
				break;
			case 'snippet':
				$this->_snippet = true;
				break;
			default:
				// Support true/false, this is here because case does a loose comparison.
				if($value === true || $value === false) {
					$this->_index = $value;
				}
		}
	}
}
