<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Core\ViewMeta;

use Core\ConfigHandler;

/**
 * Description of Title
 *
 * @author charlie
 */
class Title extends Base {
	
	public function __toString() : string {
		return $this->getSEOTitle();
	}
	
	public function getSEOTitle() : string {
		
		// The SEO title defaults back to the basic title if not set explictly.
		$title = $this->_content ?: $this->_contentKey;
		
		// Support I18N natively
		if(strpos($title, 't:') === 0){
			$title = t(substr($title, 2));
		}
		
		// Allow the admin to opt to remove "stop words" from the title.
		// These are insignificant to search engines.
		if(ConfigHandler::Get('/core/page/title_remove_stop_words')){
			$stopwords = \Core\get_stop_words();

			$exploded = explode(' ', $title);
			$nt = '';
			foreach($exploded as $w){
				$lw = strtolower($w);
				if(!in_array($lw, $stopwords)){
					$nt .= ' ' . $w;
				}
			}
			
			if(trim($nt)) {
				// Only set the title if there's still something left.
				// This allows the admin to set a title of only a single word with that word being a stopword.
				$title = trim($nt);
			}
		}
		
		
		// Support templates set from the configuration, (only if it's set)
		$template = ConfigHandler::Get('/core/page/title_template');
		if($template){
			$rep = [
				//Replaced with the title of the page
				'%%title%%' => $title,
				//The site's name
				'%%sitename%%' => SITENAME,
			];
			
			$title = str_replace(array_keys($rep), array_values($rep), $template);
		}
		
		return $title !== null ? $title : '';
	}
	
	public function getTitle() : string {
		if(strpos($this->_contentKey, 't:') === 0){
			return t(substr($this->_contentKey, 2));
		}
		else{
			return $this->_contentKey;
		}
	}
	
	public function setTitle(string $title){
		$this->_contentKey = $title;
	}
	
	public function fetch() : array {
		$html = [];
		
		$seo   = $this->getSEOTitle();
		$title = $this->getTitle();
		
		
		// Set the common title tag from either the SEO or standard title.
		if($seo){
			$html[] = '<title>' . htmlentities($seo) . '</title>';
		}
		elseif($title){
			$html[] = '<title>' . htmlentities($title) . '</title>';
		}
		
		return $html;
	}
}
