<?php
/**
 * Rewrite Map model, essentially an apache rewrite system for Core.
 *
 * This mimics the page system, with the exception that rewrite urls are the PK here,
 * and there can be multiple baseurls.  Whenever a rewriteurl changes, the previous URL is
 * saved here for lookups that are still using the previous name.
 *
 * @package Core
 * @author Charlie Powell <charlie@evalagency.com>
 * @copyright Copyright (C) 2009-2017  Charlie Powell
 * @license GNU Affero General Public License v3 <http://www.gnu.org/licenses/agpl-3.0.txt>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/agpl-3.0.txt.
 */

use Core\Model;


/**
 * Class RewriteMapModel
 *
 * @property string $baseurl
 * @property int    $created    Creation timestamp, populated automatically
 * @property bool   $fuzzy      If this url is fuzzy or an exact match
 * @property string $rewriteurl
 * @property int    $site       The site id in multisite mode, (or -1 if global)
 * @property int    $updated    Last update timestamp, populated automatically
 */
class RewriteMapModel extends Model {
	public static $ExportGroup = 't:STRING_SITE_CONTENT';

	public static $Schema = array(
		'site' => array(
			'type' => Model::ATT_TYPE_SITE,
			'formtype' => 'system',
		),
		'rewriteurl' => array(
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 96,
			'null' => false,
			'form' => array(
				'title' => 'Page URL',
				'type' => 'pagerewriteurl',
				'description' => 'Starts with a "/", omit the root web dir.',
			),
		),
		'baseurl' => array(
			'type' => Model::ATT_TYPE_STRING,
			'maxlength' => 96,
			'required' => true,
			'null' => false,
			'form' => array('type' => 'system'),
		),
		'fuzzy' => array(
			'type' => Model::ATT_TYPE_BOOL,
			'comment' => 'If this url is fuzzy or an exact match',
			'null' => false,
			'default' => '0',
			'formtype' => 'system'
		),
		'created' => array(
			'type' => Model::ATT_TYPE_CREATED,
			'null' => false,
		),
		'updated' => array(
			'type' => Model::ATT_TYPE_UPDATED,
			'null' => false,
		),
	);

	public static $Indexes = array(
		'primary' => array('rewriteurl'),
	);
}
