<?php
/**
 * Reformat the data contained within the insertables table.
 *
 * There were major changes to how they function, including
 * a switch to full JSON encoding to support multiple value elements
 * and better group handling
 */

$toDeletes = [];
$newGroups = [];

$inserts = \Core\Datamodel\Dataset::Init()
	->select('*')
	->table('insertable')
	->executeAndGet();

foreach($inserts as $row) {
	if(preg_match('/^[a-zA-Z0-9_]*\[[a-zA-Z0-9_]*\]/', $row['name'])) {
		// This is a legacy-formatted group; it'll get converted.

		$groupName = preg_replace('/^([a-zA-Z0-9_]*)\[([a-zA-Z0-9_]*)\]/', '$1', $row['name']);
		$elementName = preg_replace('/^([a-zA-Z0-9_]*)\[([a-zA-Z0-9_]*)\]/', '$2', $row['name']);
		$insertableKey = $row['site'] . ':' . $row['baseurl'] . ':' . $groupName;

		if(!isset($newGroups[ $insertableKey ])) {
			$newGroups[ $insertableKey ] = [
				'site' => $row['site'],
				'baseurl' => $row['baseurl'],
				'name' => $groupName,
				'value' => [],
			];
		}

		$groupValue = json_decode($row['value'], true);
		if(is_array($groupValue)) {
			$len = count($groupValue);
			for($i = 1, $c = 0; $i < $len; $i++, $c++) {
				if(!isset($newGroups[ $insertableKey ]['value'][ $c ])) {
					$newGroups[ $insertableKey ]['value'][ $c ] = [];
				}
				$newGroups[ $insertableKey ]['value'][ $c ][ $elementName ] = $groupValue[ $i ];
			}
		}

		$toDeletes[] = [
			'site' => $row['site'],
			'baseurl' => $row['baseurl'],
			'name' => $row['name'],
		];
	}
	else {
		// Traditional element, just check the format
		$value = json_decode($row['value'], true);
		if($value === null) {
			// Format was invalid!  Update!
			\Core\Datamodel\Dataset::Init()
				->update('value', json_encode($row['value']))
				->table('insertable')
				->whereEquals('site', $row['site'])
				->whereEquals('baseurl', $row['baseurl'])
				->whereEquals('name', $row['name'])
				->execute();
		}
	}
}

foreach($newGroups as $newGroupData) {
	if(count($newGroupData['value'])) {
		\Core\Datamodel\Dataset::Init()
			->insert([
				'site' => $newGroupData['site'],
				'baseurl' => $newGroupData['baseurl'],
				'name' => $newGroupData['name'],
				'value' => json_encode($newGroupData['value']),
			])
			->table('insertable')
			->execute();
	}
}

foreach($toDeletes as $deleteData) {
	\Core\Datamodel\Dataset::Init()
		->delete()
		->limit(1)
		->whereEquals('site', $deleteData['site'])
		->whereEquals('baseurl', $deleteData['baseurl'])
		->whereEquals('name', $deleteData['name'])
		->table('insertable')
		->execute();
}